@@ -31,7 +31,7 @@
 #include <gal/e-table/e-table-without.h>
 #include <gal/widgets/e-popup-menu.h>
 
-#include <libebook/e-book-async.h>
+#include <libebook/e-book.h>
 #include <libebook/e-contact.h>
 #include <addressbook/gui/widgets/e-addressbook-model.h>
 #include <addressbook/gui/widgets/e-addressbook-table-adapter.h>
@@ -47,10 +47,11 @@
 #include <e-util/e-categories-master-list-wombat.h>
 #include "e-util/e-sexp.h"
 
-static void e_select_names_init		(ESelectNames		 *names);
-static void e_select_names_class_init	(ESelectNamesClass	 *klass);
-static void e_select_names_dispose (GObject *object);
-static void update_query (GtkWidget *widget, ESelectNames *e_select_names);
+static void  e_select_names_init       (ESelectNames		 *names);
+static void  e_select_names_class_init (ESelectNamesClass	 *klass);
+static void  e_select_names_dispose    (GObject *object);
+static void  update_query              (GtkWidget *widget, ESelectNames *e_select_names);
+static char *get_query_string          (ESelectNames *e_select_names);
 
 static void sync_table_and_models (ESelectNamesModel *triggering_model, ESelectNames *esl);
 
@@ -120,10 +121,12 @@ search_result (EABModel *model, EBookViewStatus status, ESelectNames *esn)
 static void
 set_book(EBook *book, EBookStatus status, ESelectNames *esn)
 {
+	char *query_str = get_query_string (esn);
 	g_object_set(esn->model,
 		     "book", book,
+		     "query", query_str,
 		     NULL);
-	update_query (NULL, esn);
+	g_free (query_str);
 	g_object_unref(book);
 	g_object_unref(esn->model);
 	g_object_unref(esn);
@@ -154,12 +157,12 @@ addressbook_model_set_source (ESelectNames *e_select_names, EABModel *model, ESo
 {
 	EBook *book;
 
-	book = e_book_new();
+	book = e_book_new(source, NULL);
 
 	g_object_ref(e_select_names);
 	g_object_ref(model);
 
-	addressbook_load_source (book, source, (EBookCallback) set_book, e_select_names);
+	addressbook_load (book, (EBookCallback) set_book, e_select_names);
 }
 
 static void *
@@ -342,15 +345,15 @@ source_selected (ESourceOptionMenu *menu, ESource *source, ESelectNames *e_selec
 	e_select_names_config_set_last_completion_book (e_source_peek_uid (source));
 }
 
-static void
-update_query (GtkWidget *widget, ESelectNames *e_select_names)
+static char *
+get_query_string (ESelectNames *e_select_names)
 {
 	char *category = "";
 	const char *search = "";
-	char *query;
-	char *q_array[4];
+	EBookQuery *query;
+	EBookQuery *q_array[4];
+	char *query_str;
 	int i;
-	GString *s = g_string_new ("");
 
 	if (e_select_names->categories) {
 		category = e_categories_master_list_option_menu_get_category (E_CATEGORIES_MASTER_LIST_OPTION_MENU (e_select_names->categories));
@@ -359,34 +362,37 @@ update_query (GtkWidget *widget, ESelectNames *e_select_names)
 		search = gtk_entry_get_text (GTK_ENTRY (e_select_names->select_entry));
 	}
 
-	e_sexp_encode_string (s, search);
-
 	i = 0;
-	q_array[i++] = "(contains \"email\" \"\")";
+	q_array[i++] = e_book_query_field_test (E_CONTACT_EMAIL, E_BOOK_QUERY_CONTAINS, "");
 	if (category && *category)
-		q_array[i++] = g_strdup_printf ("(is \"category\" \"%s\")", category);
+		q_array[i++] = e_book_query_field_test (E_CONTACT_CATEGORY_LIST, E_BOOK_QUERY_IS, category);
 	if (search && *search)
-		q_array[i++] = g_strdup_printf ("(or (beginswith \"email\" %s) "
-						"    (beginswith \"full_name\" %s) "
-						"    (beginswith \"nickname\" %s)"
-						"    (beginswith \"file_as\" %s))",
-						s->str, s->str, s->str, s->str);
-	q_array[i++] = NULL;
-	if (i > 2) {
-		char *temp = g_strjoinv (" ", q_array);
-		query = g_strdup_printf ("(and %s)", temp);
-		g_free (temp);
-	} else {
-		query = g_strdup (q_array[0]);
+		q_array[i++] = e_book_query_orv (e_book_query_field_test (E_CONTACT_EMAIL, E_BOOK_QUERY_BEGINS_WITH, search),
+						 e_book_query_field_test (E_CONTACT_FULL_NAME, E_BOOK_QUERY_BEGINS_WITH, search),
+						 e_book_query_field_test (E_CONTACT_NICKNAME, E_BOOK_QUERY_BEGINS_WITH, search),
+						 e_book_query_field_test (E_CONTACT_FILE_AS, E_BOOK_QUERY_BEGINS_WITH, search),
+						 NULL);
+	if (i > 1) {
+		query = e_book_query_and (i, q_array, TRUE);
 	}
+	else {
+		query = q_array[0];
+	}
+	query_str = e_book_query_to_string (query);
+	e_book_query_unref (query);
+	return query_str;
+}
+
+
+static void
+update_query (GtkWidget *widget, ESelectNames *e_select_names)
+{
+	char *query_str = get_query_string (e_select_names);
+	printf ("query_str = %s\n", query_str);
 	g_object_set (e_select_names->model,
-		      "query", query,
+		      "query", query_str,
 		      NULL);
-	for (i = 1; q_array[i]; i++) {
-		g_free (q_array[i]);
-	}
-	g_free (query);
-	g_string_free (s, TRUE);
+	g_free (query_str);
 }
 
 static void
