@@ -27,7 +27,7 @@
 
 #include "addressbook/gui/contact-editor/eab-editor.h"
 
-#include <libebook/e-book-async.h>
+#include <libebook/e-book.h>
 #include <libebook/e-contact.h>
 
 #include <gtk/gtktreeview.h>
@@ -68,25 +68,10 @@ struct _EContactEditor
 	
 	GladeXML *gui;
 	GtkWidget *app;
-	GnomeUIInfo *email_info;
-	GnomeUIInfo *phone_info;
-	GnomeUIInfo *address_info;
-	GtkWidget *email_popup;
-	GtkWidget *phone_popup;
-	GtkWidget *address_popup;
-	GList *email_list;
-	GList *phone_list;
-	GList *address_list;
+
+	GtkWidget *file_selector;
 
 	EContactName *name;
-	char *company;
-
-	GtkListStore *im_model;
-
-	EContactField email_choice;
-	EContactField phone_choice[4];
-	EContactField address_choice;
-	EContactField address_mailing;
 
 	/* Whether we are editing a new contact or an existing one */
 	guint is_new_contact : 1;
@@ -103,12 +88,6 @@ struct _EContactEditor
 	/* Whether the contact editor will accept modifications, save */
 	guint target_editable : 1;
 
-	/* Whether the fullname will accept modifications */
-	guint fullname_editable : 1;
-
-	/* Whether each of the addresses are editable */
-	gboolean address_editable [E_CONTACT_LAST_ADDRESS_ID - E_CONTACT_FIRST_ADDRESS_ID + 1];
-
 	/* Whether an async wombat call is in progress */
 	guint in_async_call : 1;
 
@@ -117,6 +96,10 @@ struct _EContactEditor
 	/* ID for async load_source call */
 	guint  load_source_id;
 	EBook *load_book;
+
+	/* signal ids for "writable_status" */
+	int source_editable_id;
+	int target_editable_id;
 };
 
 struct _EContactEditorClass
