@@ -147,11 +147,8 @@ update_folder_bar_message (EABModel *model)
 	case 0:
 		message = g_strdup (_("No contacts"));
 		break;
-	case 1:
-		message = g_strdup (_("1 contact"));
-		break;
 	default:
-		message = g_strdup_printf (_("%d contacts"), count);
+		message = g_strdup_printf (ngettext("%d contact", "%d contacts", count), count);
 		break;
 	}
 
@@ -493,7 +490,15 @@ get_view (EABModel *model)
 	gboolean success;
 
 	if (model->book && model->query) {
-		char *query_string = e_book_query_to_string (model->query);
+		ESource *source;
+		const char *limit_str;
+		int limit = -1;
+
+		source = e_book_get_source (model->book);
+
+		limit_str = e_source_get_property (source, "limit");
+		if (limit_str && *limit_str)
+			limit = atoi (limit_str);
 
 		remove_book_view(model);
 		free_data (model);
@@ -502,23 +507,31 @@ get_view (EABModel *model)
 			model->first_get_view = FALSE;
 
 			if (e_book_check_static_capability (model->book, "do-initial-query")) {
-				success = e_book_async_get_book_view (model->book, query_string, book_view_loaded, model);
+				success = e_book_async_get_book_view (model->book, model->query, NULL, limit, book_view_loaded, model);
 			} else {
 				g_signal_emit (model,
 					       eab_model_signals [MODEL_CHANGED], 0);
 				g_signal_emit (model,
 					       eab_model_signals [STOP_STATE_CHANGED], 0);
-				g_free (query_string);
 				return;
 			}
 		}
 		else
-			success = e_book_async_get_book_view (model->book, query_string, book_view_loaded, model);
+			success = e_book_async_get_book_view (model->book, model->query, NULL, limit, book_view_loaded, model);
 
-		g_free (query_string);
 	}
 }
 
+static gboolean
+get_view_idle (EABModel *model)
+{
+	model->book_view_idle_id = 0;
+	get_view (model);
+	g_object_unref (model);
+	return FALSE;
+}
+
+
 EContact *
 eab_model_get_contact(EABModel *model,
 		      int       row)
@@ -533,6 +546,7 @@ static void
 eab_model_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	EABModel *model;
+	gboolean need_get_book_view = FALSE;
 
 	model = EAB_MODEL (object);
 	
@@ -553,6 +567,15 @@ eab_model_set_property (GObject *object, guint prop_id, const GValue *value, GPa
 		}
 		model->book = E_BOOK(g_value_get_object (value));
 		if (model->book) {
+			model->writable_status_id =
+				g_signal_connect (model->book,
+						  "writable_status",
+						  G_CALLBACK (writable_status), model);
+			model->backend_died_id =
+				g_signal_connect (model->book,
+						  "backend_died",
+						  G_CALLBACK (backend_died), model);
+
 			if (!model->editable_set) {
 				model->editable = e_book_is_writable (model->book);
 
@@ -560,22 +583,17 @@ eab_model_set_property (GObject *object, guint prop_id, const GValue *value, GPa
 					       eab_model_signals [WRITABLE_STATUS], 0,
 					       model->editable);
 			}
+
 			model->first_get_view = TRUE;
 			g_object_ref (model->book);
-			get_view (model);
-			g_signal_connect (model->book,
-					  "writable_status",
-					  G_CALLBACK (writable_status), model);
-			g_signal_connect (model->book,
-					  "backend_died",
-					  G_CALLBACK (backend_died), model);
+			need_get_book_view = TRUE;
 		}
 		break;
 	case PROP_QUERY:
 		if (model->query)
 			e_book_query_unref (model->query);
 		model->query = e_book_query_from_string (g_value_get_string (value));
-		get_view (model);
+		need_get_book_view = TRUE;
 		break;
 	case PROP_EDITABLE:
 		model->editable = g_value_get_boolean (value);
@@ -585,6 +603,14 @@ eab_model_set_property (GObject *object, guint prop_id, const GValue *value, GPa
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
+
+	if (need_get_book_view) {
+		if (!model->book_view_idle_id) {
+			g_object_ref (model);
+			model->book_view_idle_id = g_idle_add ((GSourceFunc)get_view_idle, model);
+		}
+	}
+
 }
 
 static void
