@@ -48,11 +48,13 @@
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* NONE:INT (e-calendar-marshal.list:1) */
+/* NONE:INT (./e-calendar-marshal.list:1) */
 
-/* NONE:POINTER (e-calendar-marshal.list:2) */
+/* NONE:POINTER (./e-calendar-marshal.list:2) */
 
-/* NONE:INT,STRING (e-calendar-marshal.list:3) */
+/* NONE:OBJECT (./e-calendar-marshal.list:3) */
+
+/* NONE:INT,STRING (./e-calendar-marshal.list:4) */
 void
 e_calendar_marshal_VOID__INT_STRING (GClosure     *closure,
                                      GValue       *return_value,
@@ -89,7 +91,7 @@ e_calendar_marshal_VOID__INT_STRING (GClosure     *closure,
             data2);
 }
 
-/* NONE:INT,BOOL (e-calendar-marshal.list:4) */
+/* NONE:INT,BOOL (./e-calendar-marshal.list:5) */
 void
 e_calendar_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
                                       GValue       *return_value,
@@ -126,7 +128,7 @@ e_calendar_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* NONE:INT,POINTER (e-calendar-marshal.list:5) */
+/* NONE:INT,POINTER (./e-calendar-marshal.list:6) */
 void
 e_calendar_marshal_VOID__INT_POINTER (GClosure     *closure,
                                       GValue       *return_value,
@@ -163,7 +165,44 @@ e_calendar_marshal_VOID__INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING,INT (e-calendar-marshal.list:6) */
+/* NONE:INT,OBJECT (./e-calendar-marshal.list:7) */
+void
+e_calendar_marshal_VOID__INT_OBJECT (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__INT_OBJECT) (gpointer     data1,
+                                                 gint         arg_1,
+                                                 gpointer     arg_2,
+                                                 gpointer     data2);
+  register GMarshalFunc_VOID__INT_OBJECT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__INT_OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_int (param_values + 1),
+            g_marshal_value_peek_object (param_values + 2),
+            data2);
+}
+
+/* NONE:STRING,INT (./e-calendar-marshal.list:8) */
 void
 e_calendar_marshal_VOID__STRING_INT (GClosure     *closure,
                                      GValue       *return_value,
@@ -200,7 +239,7 @@ e_calendar_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* NONE:INT,INT (e-calendar-marshal.list:7) */
+/* NONE:INT,INT (./e-calendar-marshal.list:9) */
 void
 e_calendar_marshal_VOID__INT_INT (GClosure     *closure,
                                   GValue       *return_value,
@@ -237,7 +276,7 @@ e_calendar_marshal_VOID__INT_INT (GClosure     *closure,
             data2);
 }
 
-/* NONE:ENUM,ENUM (e-calendar-marshal.list:8) */
+/* NONE:ENUM,ENUM (./e-calendar-marshal.list:10) */
 void
 e_calendar_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                     GValue       *return_value,
@@ -274,7 +313,7 @@ e_calendar_marshal_VOID__ENUM_ENUM (GClosure     *closure,
             data2);
 }
 
-/* NONE:ENUM,STRING (e-calendar-marshal.list:9) */
+/* NONE:ENUM,STRING (./e-calendar-marshal.list:11) */
 void
 e_calendar_marshal_VOID__ENUM_STRING (GClosure     *closure,
                                       GValue       *return_value,
@@ -311,7 +350,7 @@ e_calendar_marshal_VOID__ENUM_STRING (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING,BOOL,INT,INT (e-calendar-marshal.list:10) */
+/* NONE:STRING,BOOL,INT,INT (./e-calendar-marshal.list:12) */
 void
 e_calendar_marshal_VOID__STRING_BOOLEAN_INT_INT (GClosure     *closure,
                                                  GValue       *return_value,
@@ -352,7 +391,7 @@ e_calendar_marshal_VOID__STRING_BOOLEAN_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING,STRING (e-calendar-marshal.list:11) */
+/* NONE:STRING,STRING (./e-calendar-marshal.list:13) */
 void
 e_calendar_marshal_VOID__STRING_STRING (GClosure     *closure,
                                         GValue       *return_value,
@@ -389,7 +428,7 @@ e_calendar_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING,STRING,STRING (e-calendar-marshal.list:12) */
+/* NONE:STRING,STRING,STRING (./e-calendar-marshal.list:14) */
 void
 e_calendar_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                GValue       *return_value,
@@ -428,7 +467,7 @@ e_calendar_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* NONE:POINTER,ENUM (e-calendar-marshal.list:13) */
+/* NONE:POINTER,ENUM (./e-calendar-marshal.list:15) */
 void
 e_calendar_marshal_VOID__POINTER_ENUM (GClosure     *closure,
                                        GValue       *return_value,
@@ -465,7 +504,7 @@ e_calendar_marshal_VOID__POINTER_ENUM (GClosure     *closure,
             data2);
 }
 
-/* NONE:POINTER,STRING (e-calendar-marshal.list:14) */
+/* NONE:POINTER,STRING (./e-calendar-marshal.list:16) */
 void
 e_calendar_marshal_VOID__POINTER_STRING (GClosure     *closure,
                                          GValue       *return_value,
@@ -502,7 +541,7 @@ e_calendar_marshal_VOID__POINTER_STRING (GClosure     *closure,
             data2);
 }
 
-/* NONE:POINTER,POINTER (e-calendar-marshal.list:15) */
+/* NONE:POINTER,POINTER (./e-calendar-marshal.list:17) */
 void
 e_calendar_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                           GValue       *return_value,
@@ -539,3 +578,40 @@ e_calendar_marshal_VOID__POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
+/* NONE:LONG,LONG (./e-calendar-marshal.list:18) */
+void
+e_calendar_marshal_VOID__LONG_LONG (GClosure     *closure,
+                                    GValue       *return_value,
+                                    guint         n_param_values,
+                                    const GValue *param_values,
+                                    gpointer      invocation_hint,
+                                    gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__LONG_LONG) (gpointer     data1,
+                                                glong        arg_1,
+                                                glong        arg_2,
+                                                gpointer     data2);
+  register GMarshalFunc_VOID__LONG_LONG callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__LONG_LONG) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_long (param_values + 1),
+            g_marshal_value_peek_long (param_values + 2),
+            data2);
+}
+
