@@ -214,13 +214,14 @@ struct _GpgCtx {
 	unsigned int goodsig:1;
 	unsigned int validsig:1;
 	unsigned int nopubkey:1;
+	unsigned int nodata:1;
 	unsigned int trust:3;
 	
 	unsigned int diagflushed:1;
 	
 	unsigned int utf8:1;
 	
-	unsigned int padding:11;
+	unsigned int padding:10;
 };
 
 static struct _GpgCtx *
@@ -265,6 +266,7 @@ gpg_ctx_new (CamelSession *session)
 	gpg->need_id = NULL;
 	gpg->passwd = NULL;
 	
+	gpg->nodata = FALSE;
 	gpg->hadsig = FALSE;
 	gpg->badsig = FALSE;
 	gpg->errsig = FALSE;
@@ -713,7 +715,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 	*inptr++ = '\0';
 	status = gpg->statusbuf;
 	
-	d(printf ("status: %s\n", status));
+	printf ("status: %s\n", status);
 	
 	if (strncmp (status, "[GNUPG:] ", 9) != 0) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -772,7 +774,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		prompt = g_strdup_printf (_("You need a passphrase to unlock the key for\n"
 					    "user: \"%s\""), name);
 		
-		if ((passwd = camel_session_get_password (gpg->session, prompt, CAMEL_SESSION_PASSWORD_SECRET, NULL, gpg->need_id, ex)) && !gpg->utf8) {
+		if ((passwd = camel_session_get_password (gpg->session, NULL, NULL, prompt,  gpg->need_id, CAMEL_SESSION_PASSWORD_SECRET, ex)) && !gpg->utf8) {
 			char *opasswd = passwd;
 			
 			if ((passwd = g_locale_to_utf8 (passwd, -1, &nread, &nwritten, NULL))) {
@@ -800,7 +802,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 	} else if (!strncmp (status, "BAD_PASSPHRASE", 14)) {
 		gpg->bad_passwds++;
 		
-		camel_session_forget_password (gpg->session, NULL, gpg->need_id, ex);
+		camel_session_forget_password (gpg->session, NULL, NULL, gpg->need_id, ex);
 		
 		if (gpg->bad_passwds == 3) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
@@ -816,6 +818,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 	} else if (!strncmp (status, "NODATA", 6)) {
 		/* this is an error */
 		/* But we ignore it anyway, we should get other response codes to say why */
+		gpg->nodata = TRUE;
 	} else {
 		/* check to see if we are complete */
 		switch (gpg->mode) {
@@ -1382,7 +1385,7 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 	struct _GpgCtx *gpg = NULL;
 	char *sigfile = NULL;
 	CamelContentType *ct;
-	CamelMimePart *sigpart, *datapart;
+	CamelMimePart *sigpart;
 	CamelStream *istream = NULL;
 	CamelMultipart *mps;
 
@@ -1398,15 +1401,19 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 		return NULL;
 	}
 
-	datapart = camel_multipart_get_part(mps, CAMEL_MULTIPART_SIGNED_CONTENT);
-	sigpart = camel_multipart_get_part(mps, CAMEL_MULTIPART_SIGNED_SIGNATURE);
-
-	if (sigpart == NULL || datapart == NULL) {
+	if (!(istream = camel_multipart_signed_get_content_stream ((CamelMultipartSigned *) mps, NULL))) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot verify message signature: Incorrect message format"));
-		goto exception;
+		return NULL;
 	}
-
+	
+	if (!(sigpart = camel_multipart_get_part (mps, CAMEL_MULTIPART_SIGNED_SIGNATURE))) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Cannot verify message signature: Incorrect message format"));
+		camel_object_unref (istream);
+		return NULL;
+	}
+	
 	sigfile = swrite (sigpart);
 	if (sigfile == NULL) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -1414,11 +1421,8 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 				      g_strerror (errno));
 		goto exception;
 	}
-
-	istream = camel_stream_mem_new();
-	camel_cipher_canonical_to_stream(datapart, CAMEL_MIME_FILTER_CANON_CRLF, istream);
-	camel_stream_reset(istream);
 	
+	camel_stream_reset(istream);
 	gpg = gpg_ctx_new (context->session);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_VERIFY);
 	gpg_ctx_set_hash (gpg, camel_cipher_id_to_hash(context, camel_content_type_param(ct, "micalg")));
@@ -1428,7 +1432,6 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 	if (gpg_ctx_op_start (gpg) == -1) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
 				     _("Failed to execute gpg."));
-		gpg_ctx_free (gpg);
 		goto exception;
 	}
 	
@@ -1457,8 +1460,6 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 			validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_GOOD;
 		else
 			validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
-	} else if (gpg->nopubkey) {
-		validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_UNKNOWN;
 	} else {
 		validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
 	}
@@ -1474,8 +1475,9 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 	
  exception:
 	
-	gpg_ctx_free (gpg);
-
+	if (gpg != NULL)
+		gpg_ctx_free (gpg);
+	
 	if (istream)
 		camel_object_unref(istream);
 
