@@ -86,7 +86,7 @@ static gboolean smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *mess
 static gboolean smtp_rset (CamelSmtpTransport *transport, CamelException *ex);
 static gboolean smtp_quit (CamelSmtpTransport *transport, CamelException *ex);
 
-static void smtp_set_exception (CamelSmtpTransport *transport, const char *respbuf,
+static void smtp_set_exception (CamelSmtpTransport *transport, gboolean disconnect, const char *respbuf,
 				const char *message, CamelException *ex);
 
 /* private data members */
@@ -281,7 +281,7 @@ connect_to_server (CamelService *service, int try_starttls, CamelException *ex)
 	ret = camel_tcp_stream_connect (CAMEL_TCP_STREAM (tcp_stream), h, port);
 	camel_free_host (h);
 	if (ret == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 				      _("Could not connect to %s (port %d): %s"),
 				      service->url->host, port,
 				      g_strerror (errno));
@@ -305,7 +305,7 @@ connect_to_server (CamelService *service, int try_starttls, CamelException *ex)
 		g_free (respbuf);
 		respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
 		if (!respbuf || strncmp (respbuf, "220", 3)) {
-			smtp_set_exception (transport, respbuf,  _("Welcome response error"), ex);
+			smtp_set_exception (transport, FALSE, respbuf,  _("Welcome response error"), ex);
 			g_free (respbuf);
 			return FALSE;
 		}
@@ -354,8 +354,8 @@ connect_to_server (CamelService *service, int try_starttls, CamelException *ex)
  starttls:
 	d(fprintf (stderr, "sending : STARTTLS\r\n"));
 	if (camel_stream_write (tcp_stream, "STARTTLS\r\n", 10) == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("STARTTLS request timed out: %s"),
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("STARTTLS command failed: %s"),
 				      g_strerror (errno));
 		goto exception_cleanup;
 	}
@@ -370,7 +370,7 @@ connect_to_server (CamelService *service, int try_starttls, CamelException *ex)
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "220", 3)) {
-			smtp_set_exception (transport, respbuf, _("STARTTLS response error"), ex);
+			smtp_set_exception (transport, FALSE, respbuf, _("STARTTLS command failed"), ex);
 			g_free (respbuf);
 			goto exception_cleanup;
 		}
@@ -503,7 +503,7 @@ smtp_connect (CamelService *service, CamelException *ex)
 		while (!authenticated) {
 			if (errbuf) {
 				/* We need to un-cache the password before prompting again */
-				camel_session_forget_password (session, service, "password", NULL);
+				camel_session_forget_password (session, service, NULL, "password", NULL);
 				g_free (service->url->passwd);
 				service->url->passwd = NULL;
 			}
@@ -515,8 +515,8 @@ smtp_connect (CamelService *service, CamelException *ex)
 							  errbuf ? errbuf : "", service->url->user,
 							  service->url->host);
 				
-				service->url->passwd = camel_session_get_password (session, prompt, CAMEL_SESSION_PASSWORD_SECRET,
-										   service, "password", ex);
+				service->url->passwd = camel_session_get_password (session, service, NULL,
+										   prompt, "password", CAMEL_SESSION_PASSWORD_SECRET, ex);
 				
 				g_free (prompt);
 				g_free (errbuf);
@@ -536,15 +536,6 @@ smtp_connect (CamelService *service, CamelException *ex)
 				camel_exception_clear (ex);
 			}
 		}
-		
-		/* The spec says we have to re-EHLO, but some servers
-		 * we won't bother to name don't want you to... so ignore
-		 * errors.
-		 */
-		if (!smtp_helo (transport, ex) && !transport->connected)
-			return FALSE;
-		
-		camel_exception_clear (ex);
 	}
 	
 	return TRUE;
@@ -681,10 +672,15 @@ smtp_send_to (CamelTransport *transport, CamelMimeMessage *message,
 	const char *addr;
 	int i, len;
 	
+	if (!smtp_transport->connected) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
+				      _("Cannot send message: service not connected."));
+		return FALSE;
+	}
+	
 	if (!camel_internet_address_get (CAMEL_INTERNET_ADDRESS (from), 0, NULL, &addr)) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot send message: "
-					"sender address not valid."));
+				      _("Cannot send message: sender address not valid."));
 		return FALSE;
 	}
 	
@@ -711,14 +707,14 @@ smtp_send_to (CamelTransport *transport, CamelMimeMessage *message,
 	cia = CAMEL_INTERNET_ADDRESS (recipients);
 	for (i = 0; i < len; i++) {
 		char *enc;
-
+		
 		if (!camel_internet_address_get (cia, i, NULL, &addr)) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
 					     _("Cannot send message: one or more invalid recipients"));
 			camel_operation_end (NULL);
 			return FALSE;
 		}
-
+		
 		enc = camel_internet_address_encode_address(NULL, NULL, addr);
 		if (!smtp_rcpt (smtp_transport, enc, ex)) {
 			g_free(enc);
@@ -734,7 +730,8 @@ smtp_send_to (CamelTransport *transport, CamelMimeMessage *message,
 	}
 	
 	/* reset the service for our next transfer session */
-	smtp_rset (smtp_transport, ex);
+	if (!smtp_rset (smtp_transport, ex))
+		camel_exception_clear (ex);
 	
 	camel_operation_end (NULL);
 	
@@ -808,7 +805,7 @@ smtp_decode_status_code (const char *in, size_t len)
 }
 
 static void
-smtp_set_exception (CamelSmtpTransport *transport, const char *respbuf, const char *message, CamelException *ex)
+smtp_set_exception (CamelSmtpTransport *transport, gboolean disconnect, const char *respbuf, const char *message, CamelException *ex)
 {
 	const char *token, *rbuf = respbuf;
 	char *buffer = NULL;
@@ -818,8 +815,8 @@ smtp_set_exception (CamelSmtpTransport *transport, const char *respbuf, const ch
 	if (!respbuf || !(transport->flags & CAMEL_SMTP_TRANSPORT_ENHANCEDSTATUSCODES)) {
 	fake_status_code:
 		error = respbuf ? atoi (respbuf) : 0;
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, "%s: %s", message,
-				      smtp_error_string (error));
+		camel_exception_setv (ex, error == 0 && errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      "%s: %s", message, smtp_error_string (error));
 	} else {
 		string = g_string_new ("");
 		do {
@@ -856,7 +853,10 @@ smtp_set_exception (CamelSmtpTransport *transport, const char *respbuf, const ch
 	
 	if (!respbuf) {
 		/* we got disconnected */
-		transport->connected = FALSE;
+		if (disconnect)
+			camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
+		else
+			transport->connected = FALSE;
 	}
 }
 
@@ -930,16 +930,11 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 	d(fprintf (stderr, "sending : %s", cmdbuf));
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("HELO request timed out: %s"),
-				      g_strerror (errno));
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("HELO command failed: %s"), g_strerror (errno));
 		camel_operation_end (NULL);
 		
-		transport->connected = FALSE;
-		camel_object_unref (transport->istream);
-		transport->istream = NULL;
-		camel_object_unref (transport->ostream);
-		transport->ostream = NULL;
+		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
 		
 		return FALSE;
 	}
@@ -953,7 +948,7 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
-			smtp_set_exception (transport, respbuf, _("HELO response error"), ex);
+			smtp_set_exception (transport, FALSE, respbuf, _("HELO command failed"), ex);
 			camel_operation_end (NULL);
 			g_free (respbuf);
 			
@@ -1041,9 +1036,8 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 	d(fprintf (stderr, "sending : %s", cmdbuf));
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("AUTH request timed out: %s"),
-				      g_strerror (errno));
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("AUTH command failed: %s"), g_strerror (errno));
 		goto lose;
 	}
 	g_free (cmdbuf);
@@ -1053,17 +1047,15 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 	
 	while (!camel_sasl_authenticated (sasl)) {
 		if (!respbuf) {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("AUTH request timed out: %s"),
-					      g_strerror (errno));
+			camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+					      _("AUTH command failed: %s"), g_strerror (errno));
 			goto lose;
 		}
 		
 		/* the server challenge/response should follow a 334 code */
-		if (strncmp (respbuf, "334", 3)) {
+		if (strncmp (respbuf, "334", 3) != 0) {
+			smtp_set_exception (transport, FALSE, respbuf, _("AUTH command failed"), ex);
 			g_free (respbuf);
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("AUTH request failed."));
 			goto lose;
 		}
 		
@@ -1148,14 +1140,11 @@ smtp_mail (CamelSmtpTransport *transport, const char *sender, gboolean has_8bit_
 	
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("MAIL FROM request timed out: %s: mail not sent"),
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("MAIL FROM command failed: %s: mail not sent"),
 				      g_strerror (errno));
 		
-		camel_object_unref (transport->istream);
-		transport->istream = NULL;
-		camel_object_unref (transport->ostream);
-		transport->ostream = NULL;
+		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
 		
 		return FALSE;
 	}
@@ -1169,7 +1158,7 @@ smtp_mail (CamelSmtpTransport *transport, const char *sender, gboolean has_8bit_
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
-			smtp_set_exception (transport, respbuf, _("MAIL FROM response error"), ex);
+			smtp_set_exception (transport, TRUE, respbuf, _("MAIL FROM command failed"), ex);
 			g_free (respbuf);
 			return FALSE;
 		}
@@ -1192,14 +1181,11 @@ smtp_rcpt (CamelSmtpTransport *transport, const char *recipient, CamelException
 	
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("RCPT TO request timed out: %s: mail not sent"),
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("RCPT TO command failed: %s: mail not sent"),
 				      g_strerror (errno));
 		
-		camel_object_unref (transport->istream);
-		transport->istream = NULL;
-		camel_object_unref (transport->ostream);
-		transport->ostream = NULL;
+		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
 		
 		return FALSE;
 	}
@@ -1216,7 +1202,7 @@ smtp_rcpt (CamelSmtpTransport *transport, const char *recipient, CamelException
 			char *message;
 			
 			message = g_strdup_printf (_("RCPT TO <%s> failed"), recipient);
-			smtp_set_exception (transport, respbuf, message, ex);
+			smtp_set_exception (transport, TRUE, respbuf, message, ex);
 			g_free (message);
 			g_free (respbuf);
 			return FALSE;
@@ -1253,14 +1239,11 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 	
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("DATA request timed out: %s: mail not sent"),
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("DATA command failed: %s: mail not sent"),
 				      g_strerror (errno));
 		
-		camel_object_unref (transport->istream);
-		transport->istream = NULL;
-		camel_object_unref (transport->ostream);
-		transport->ostream = NULL;
+		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
 		
 		return FALSE;
 	}
@@ -1274,7 +1257,7 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 		/* we should have gotten instructions on how to use the DATA command:
 		 * 354 Enter mail, end with "." on a line by itself
 		 */
-		smtp_set_exception (transport, respbuf, _("DATA response error"), ex);
+		smtp_set_exception (transport, TRUE, respbuf, _("DATA command failed"), ex);
 		g_free (respbuf);
 		return FALSE;
 	}
@@ -1295,7 +1278,7 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 	header = (struct _camel_header_raw *) &CAMEL_MIME_PART (message)->headers;
 	n = header->next;
 	while (n != NULL) {
-		if (!strcasecmp (n->name, "Bcc")) {
+		if (!g_ascii_strcasecmp (n->name, "Bcc")) {
 			header->next = n->next;
 			tail->next = n;
 			n->next = NULL;
@@ -1314,17 +1297,13 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 	header->next = savedbcc;
 	
 	if (ret == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("DATA send timed out: message termination: "
-					"%s: mail not sent"),
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("DATA command failed: %s: mail not sent"),
 				      g_strerror (errno));
 		
 		camel_object_unref (filtered_stream);
 		
-		camel_object_unref (transport->istream);
-		transport->istream = NULL;
-		camel_object_unref (transport->ostream);
-		transport->ostream = NULL;
+		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
 		
 		return FALSE;
 	}
@@ -1337,15 +1316,11 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 	d(fprintf (stderr, "sending : \\r\\n.\\r\\n\n"));
 	
 	if (camel_stream_write (transport->ostream, "\r\n.\r\n", 5) == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("DATA send timed out: message termination: "
-					"%s: mail not sent"),
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("DATA command failed: %s: mail not sent"),
 				      g_strerror (errno));
 		
-		camel_object_unref (transport->istream);
-		transport->istream = NULL;
-		camel_object_unref (transport->ostream);
-		transport->ostream = NULL;
+		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
 		
 		return FALSE;
 	}
@@ -1358,7 +1333,7 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
-			smtp_set_exception (transport, respbuf, _("DATA termination response error"), ex);
+			smtp_set_exception (transport, TRUE, respbuf, _("DATA command failed"), ex);
 			g_free (respbuf);
 			return FALSE;
 		}
@@ -1380,14 +1355,10 @@ smtp_rset (CamelSmtpTransport *transport, CamelException *ex)
 	
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("RSET request timed out: %s"),
-				      g_strerror (errno));
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("RSET command failed: %s"), g_strerror (errno));
 		
-		camel_object_unref (transport->istream);
-		transport->istream = NULL;
-		camel_object_unref (transport->ostream);
-		transport->ostream = NULL;
+		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
 		
 		return FALSE;
 	}
@@ -1401,7 +1372,7 @@ smtp_rset (CamelSmtpTransport *transport, CamelException *ex)
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
-			smtp_set_exception (transport, respbuf, _("RSET response error"), ex);
+			smtp_set_exception (transport, TRUE, respbuf, _("RSET command failed"), ex);
 			g_free (respbuf);
 			return FALSE;
 		}
@@ -1423,14 +1394,8 @@ smtp_quit (CamelSmtpTransport *transport, CamelException *ex)
 	
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("QUIT request timed out: %s"),
-				      g_strerror (errno));
-		
-		camel_object_unref (transport->istream);
-		transport->istream = NULL;
-		camel_object_unref (transport->ostream);
-		transport->ostream = NULL;
+		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
+				      _("QUIT command failed: %s"), g_strerror (errno));
 		
 		return FALSE;
 	}
@@ -1444,7 +1409,7 @@ smtp_quit (CamelSmtpTransport *transport, CamelException *ex)
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "221", 3)) {
-			smtp_set_exception (transport, respbuf, _("QUIT response error"), ex);
+			smtp_set_exception (transport, FALSE, respbuf, _("QUIT command failed"), ex);
 			g_free (respbuf);
 			return FALSE;
 		}
