@@ -35,6 +35,7 @@
 #include <camel/camel.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtktogglebutton.h>
+#include <gtk/gtkdialog.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
 #include "e-util/e-mktemp.h"
@@ -186,23 +187,22 @@ e_msg_composer_attachment_new (const char *file_name,
 		return NULL;
 	}
 	
-	wrapper = camel_data_wrapper_new ();
-	camel_data_wrapper_construct_from_stream (wrapper, stream);
-	
 	mime_type = e_msg_composer_guess_mime_type (file_name);
 	if (mime_type) {
-		if (!strcasecmp (mime_type, "message/rfc822")) {
-			camel_object_unref (wrapper);
+		if (!g_ascii_strcasecmp (mime_type, "message/rfc822")) {
 			wrapper = (CamelDataWrapper *) camel_mime_message_new ();
-			
-			camel_stream_reset (stream);
-			camel_data_wrapper_construct_from_stream (wrapper, stream);
+		} else {
+			wrapper = camel_data_wrapper_new ();
 		}
 		
+		camel_data_wrapper_construct_from_stream (wrapper, stream);
 		camel_data_wrapper_set_mime_type (wrapper, mime_type);
 		g_free (mime_type);
-	} else
+	} else {
+		wrapper = camel_data_wrapper_new ();
+		camel_data_wrapper_construct_from_stream (wrapper, stream);
 		camel_data_wrapper_set_mime_type (wrapper, "application/octet-stream");
+	}
 	
 	camel_object_unref (stream);
 	
@@ -211,7 +211,7 @@ e_msg_composer_attachment_new (const char *file_name,
 	camel_object_unref (wrapper);
 	
 	camel_mime_part_set_disposition (part, disposition);
-	filename = g_path_get_basename(file_name);
+	filename = g_path_get_basename (file_name);
 	camel_mime_part_set_filename (part, filename);
 	g_free (filename);
 	
@@ -221,7 +221,7 @@ e_msg_composer_attachment_new (const char *file_name,
            workaround, don't set a Content-Id on these parts. Fixes
            bug #10032 */
 	/* set the Content-Id */
-	content_id = header_msgid_generate ();
+	content_id = camel_header_msgid_generate ();
 	camel_mime_part_set_content_id (part, content_id);
 	g_free (content_id);
 #endif
@@ -301,21 +301,6 @@ destroy_dialog_data (DialogData *data)
  * consistent and possibly check for errors somewhere.
  */
 
-static void
-update_mime_type (DialogData *data)
-{
-	const char *filename, *mime_type;
-	
-	if (!data->attachment->guessed_type)
-		return;
-	
-	filename = gtk_entry_get_text (data->file_name_entry);
-	if (filename) {
-		if ((mime_type = gnome_vfs_mime_type_from_name (filename)))
-			gtk_entry_set_text (data->mime_type_entry, mime_type);
-	}
-}
-
 static void
 set_entry (GladeXML *xml, const char *widget_name, const char *value)
 {
@@ -393,18 +378,15 @@ ok_cb (GtkWidget *widget, gpointer data)
 	close_cb (widget, data);
 }
 
-static gboolean
-file_name_focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
+static void
+response_cb (GtkWidget *widget, gint response, gpointer data)
 {
-	DialogData *dialog_data;
-	
-	dialog_data = (DialogData *) data;
-	update_mime_type (dialog_data);
-	
-	return FALSE;
+	if (response == GTK_RESPONSE_OK)
+		ok_cb (widget, data);
+	else
+		close_cb (widget, data);
 }
 
-
 void
 e_msg_composer_attachment_edit (EMsgComposerAttachment *attachment, GtkWidget *parent)
 {
@@ -457,7 +439,7 @@ e_msg_composer_attachment_edit (EMsgComposerAttachment *attachment, GtkWidget *p
 	set_entry (editor_gui, "description_entry",
 		   camel_mime_part_get_description (attachment->body));
 	content_type = camel_mime_part_get_content_type (attachment->body);
-	type = header_content_type_simple (content_type);
+	type = camel_content_type_simple (content_type);
 	set_entry (editor_gui, "mime_type_entry", type);
 	g_free (type);
 	
@@ -465,12 +447,7 @@ e_msg_composer_attachment_edit (EMsgComposerAttachment *attachment, GtkWidget *p
 	gtk_toggle_button_set_active (dialog_data->disposition_checkbox,
 				      disposition && !g_ascii_strcasecmp (disposition, "inline"));
 	
-	connect_widget (editor_gui, "ok_button", "clicked", (GCallback)ok_cb, dialog_data);
-	connect_widget (editor_gui, "close_button", "clicked", (GCallback)close_cb, dialog_data);
-	
-	connect_widget (editor_gui, "file_name_entry", "focus-out-event",
-			(GCallback)file_name_focus_out_cb, dialog_data);
-	
+	connect_widget (editor_gui, "dialog", "response", (GCallback)response_cb, dialog_data);
 #warning "signal connect while alive"	
 	/* make sure that when the composer gets hidden/closed that our windows also close */
 	parent = gtk_widget_get_toplevel (parent);
