@@ -21,11 +21,14 @@
  *
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include <string.h>
 
 #include <gtk/gtkvbox.h>
@@ -44,15 +47,9 @@
 #include "mail-config.h"
 
 #include <e-util/e-dialog-utils.h>
+#include <e-util/e-icon-factory.h>
 
-/*#include <camel/camel-mime-message.h>*/
 #include <camel/camel-stream.h>
-/*#include <camel/camel-stream-filter.h>
-#include <camel/camel-mime-filter.h>
-#include <camel/camel-mime-filter-tohtml.h>
-#include <camel/camel-mime-filter-enriched.h>
-#include <camel/camel-multipart.h>
-#include <camel/camel-stream-mem.h>*/
 #include <camel/camel-url.h>
 
 #include <bonobo/bonobo-main.h>
@@ -65,18 +62,13 @@
 /* for efilterbar stuff */
 #include <e-util/e-sexp.h>
 #include "mail-vfolder.h"
-#include "filter/vfolder-rule.h"
+#include "em-vfolder-rule.h"
 #include <widgets/misc/e-filter-bar.h>
 #include <camel/camel-search-private.h>
 
-/* gal view crap */
-#include <gal/menus/gal-view-etable.h>
-#include <gal/menus/gal-view-instance.h>
-#include <gal/menus/gal-view-factory-etable.h>
-#include "widgets/menus/gal-view-menus.h"
-
 #include "e-util/e-dialog-utils.h"
 #include "em-utils.h"
+#include "em-composer-utils.h"
 #include "em-format-html-display.h"
 #include "em-format-html-print.h"
 #include "em-folder-browser.h"
@@ -96,13 +88,14 @@ struct _EMFolderBrowserPrivate {
 
 	GtkWidget *subscribe_editor;
 
-	GalViewInstance *view_instance;
-	GalViewMenus *view_menus;
-
 	guint search_menu_activated_id;
 	guint search_activated_id;
 	guint search_query_changed_id;
-
+	
+	double default_scroll_position;
+	guint idle_scroll_id;
+	guint list_scrolled_id;
+	
 	guint vpane_resize_id;
 	guint list_built_id;	/* hook onto list-built for delayed 'select first unread' stuff */
 	
@@ -121,8 +114,6 @@ static void emfb_search_query_changed(ESearchBar *esb, EMFolderBrowser *emfb);
 static int emfb_list_key_press(ETree *tree, int row, ETreePath path, int col, GdkEvent *ev, EMFolderBrowser *emfb);
 static void emfb_list_message_selected (MessageList *ml, const char *uid, EMFolderBrowser *emfb);
 
-static void emfb_create_view_menus(EMFolderBrowser *emfb, BonoboUIComponent *uic);
-
 static const EMFolderViewEnable emfb_enable_map[];
 
 enum {
@@ -139,9 +130,6 @@ static ESearchBarItem emfb_search_items[] = {
 	{ NULL, -1, NULL }
 };
 
-static GalViewCollection *collection = NULL;
-
-
 static EMFolderViewClass *emfb_parent;
 
 /* Needed since the paned wont take the position its given otherwise ... */
@@ -176,6 +164,7 @@ emfb_init(GObject *o)
 	p = emfb->priv = g_malloc0(sizeof(struct _EMFolderBrowserPrivate));
 
 	emfb->view.preview_active = TRUE;
+	emfb->view.list_active = TRUE;
 
 	g_slist_free(emfb->view.ui_files);
 	emfb->view.ui_files = g_slist_append(NULL, EVOLUTION_UIDIR "/evolution-mail-global.xml");
@@ -246,7 +235,17 @@ emfb_destroy(GtkObject *o)
 		g_signal_handler_disconnect(((EMFolderView *)emfb)->list, emfb->priv->list_built_id);
 		emfb->priv->list_built_id = 0;
 	}
-
+	
+	if (emfb->priv->list_scrolled_id) {
+		g_signal_handler_disconnect (((EMFolderView *) emfb)->list, emfb->priv->list_scrolled_id);
+		emfb->priv->list_scrolled_id = 0;
+	}
+	
+	if (emfb->priv->idle_scroll_id) {
+		g_source_remove (emfb->priv->idle_scroll_id);
+		emfb->priv->idle_scroll_id = 0;
+	}
+	
 	((GtkObjectClass *)emfb_parent)->destroy(o);
 }
 
@@ -254,6 +253,7 @@ static void
 emfb_class_init(GObjectClass *klass)
 {
 	klass->finalize = emfb_finalise;
+
 	((GtkObjectClass *)klass)->destroy = emfb_destroy;
 	((EMFolderViewClass *)klass)->set_folder = emfb_set_folder;
 	((EMFolderViewClass *)klass)->activate = emfb_activate;
@@ -309,7 +309,7 @@ void em_folder_browser_show_preview(EMFolderBrowser *emfb, gboolean state)
 			char *uid = g_alloca(strlen(emfb->view.list->cursor_uid)+1);
 
 			strcpy(uid, emfb->view.list->cursor_uid);
-			em_folder_view_set_message(&emfb->view, uid);
+			em_folder_view_set_message(&emfb->view, uid, FALSE);
 		}
 
 		/* need to load/show the current message? */
@@ -356,8 +356,8 @@ emfb_search_menu_activated(ESearchBar *esb, int id, EMFolderBrowser *emfb)
 			g_free (name);
 			
 			filter_rule_set_source(rule, FILTER_SOURCE_INCOMING);
-			vfolder_rule_add_source((VfolderRule *)rule, emfb->view.folder_uri);
-			vfolder_gui_add_rule((VfolderRule *)rule);
+			em_vfolder_rule_add_source((EMVFolderRule *)rule, emfb->view.folder_uri);
+			vfolder_gui_add_rule((EMVFolderRule *)rule);
 		}
 		break;
 	}
@@ -489,10 +489,10 @@ emfb_edit_cut(BonoboUIComponent *uid, void *data, const char *path)
 
 	if (GTK_WIDGET_HAS_FOCUS(((ESearchBar *)emfb->search)->entry))
 		gtk_editable_cut_clipboard((GtkEditable *)((ESearchBar *)emfb->search)->entry);
-	else if (message_list_has_primary_selection(emfb->view.list))
-		message_list_copy(emfb->view.list, TRUE);
-	else if (emfb->view.preview_active)
+	else if (GTK_WIDGET_HAS_FOCUS(emfb->view.preview->formathtml.html))
 		em_format_html_display_cut(emfb->view.preview);
+	else
+		message_list_copy(emfb->view.list, TRUE);
 }
 
 static void
@@ -502,10 +502,10 @@ emfb_edit_copy(BonoboUIComponent *uid, void *data, const char *path)
 
 	if (GTK_WIDGET_HAS_FOCUS(((ESearchBar *)emfb->search)->entry))
 		gtk_editable_copy_clipboard((GtkEditable *)((ESearchBar *)emfb->search)->entry);
-	else if (message_list_has_primary_selection(emfb->view.list))
-		message_list_copy(emfb->view.list, FALSE);
-	else if (emfb->view.preview_active)
+	else if (GTK_WIDGET_HAS_FOCUS(emfb->view.preview->formathtml.html))
 		em_format_html_display_copy(emfb->view.preview);
+	else
+		message_list_copy(emfb->view.list, FALSE);
 }
 
 static void
@@ -621,16 +621,18 @@ emfb_empty_trash(BonoboUIComponent *uid, void *data, const char *path)
 static void
 emfb_mail_compose(BonoboUIComponent *uid, void *data, const char *path)
 {
-	if (!em_utils_check_user_can_send_mail((GtkWidget *)data))
+	EMFolderView *emfv = data;
+
+	if (!em_utils_check_user_can_send_mail((GtkWidget *)emfv))
 		return;
 
-	em_utils_compose_new_message ();
+	em_utils_compose_new_message(emfv->folder_uri);
 }
 
 static void
 emfb_mail_stop(BonoboUIComponent *uid, void *data, const char *path)
 {
-	camel_operation_cancel(NULL);
+	mail_cancel_all();
 }
 
 static void
@@ -706,12 +708,12 @@ static BonoboUIVerb emfb_verbs[] = {
 };
 
 static EPixmap emfb_pixmaps[] = {
-	E_PIXMAP ("/commands/ChangeFolderProperties", "configure_16_folder.xpm"),
-	E_PIXMAP ("/commands/ViewHideRead", "hide_read_messages.xpm"),
-	E_PIXMAP ("/commands/ViewHideSelected", "hide_selected_messages.xpm"),
-	E_PIXMAP ("/commands/ViewShowAll", "show_all_messages.xpm"),
+	E_PIXMAP ("/commands/ChangeFolderProperties", "stock_folder-properties", E_ICON_SIZE_MENU),
+	E_PIXMAP ("/commands/ViewHideRead", "stock_mail-hide-read", E_ICON_SIZE_MENU),
+	E_PIXMAP ("/commands/ViewHideSelected", "stock_mail-hide-selected", E_ICON_SIZE_MENU),
+	E_PIXMAP ("/commands/ViewShowAll", "stock_show-all", E_ICON_SIZE_MENU),
 	
-	E_PIXMAP ("/commands/MailCompose", "new-message.xpm"),
+	E_PIXMAP ("/commands/MailCompose", "stock_mail-compose", E_ICON_SIZE_MENU),
 
 	E_PIXMAP_END
 };
@@ -738,10 +740,7 @@ emfb_hide_deleted(BonoboUIComponent *uic, const char *path, Bonobo_UIComponent_E
 
 	gconf = mail_config_get_gconf_client ();
 	gconf_client_set_bool(gconf, "/apps/evolution/mail/display/show_deleted", state[0] == '0', NULL);
-	if (!(emfv->folder && (emfv->folder->folder_flags & CAMEL_FOLDER_IS_TRASH))) {
-		message_list_set_hidedeleted(emfv->list, state[0] != '0');
-		emfv->hide_deleted = state[0] != '0';
-	}
+	em_folder_view_set_hide_deleted(emfv, state[0] != '0');
 }
 
 static void
@@ -784,133 +783,78 @@ emfb_view_preview(BonoboUIComponent *uic, const char *path, Bonobo_UIComponent_E
 	em_folder_browser_show_preview((EMFolderBrowser *)emfv, state[0] != '0');
 }
 
+static void
+emfb_list_scrolled (MessageList *ml, EMFolderBrowser *emfb)
+{
+	EMFolderView *emfv = (EMFolderView *) emfb;
+	double position;
+	char *state;
+	
+	position = message_list_get_scrollbar_position (ml);
+	state = g_strdup_printf ("%f", position);
+	
+	if (camel_object_meta_set (emfv->folder, "evolution:list_scroll_position", state))
+		camel_object_state_write (emfv->folder);
+	
+	g_free (state);
+}
+
+static gboolean
+scroll_idle_cb (EMFolderBrowser *emfb)
+{
+	EMFolderView *emfv = (EMFolderView *) emfb;
+	double position;
+	char *state;
+	
+	if ((state = camel_object_meta_get (emfv->folder, "evolution:list_scroll_position"))) {
+		position = strtod (state, NULL);
+		g_free (state);
+	} else {
+		position = emfb->priv->default_scroll_position;
+	}
+	
+	message_list_set_scrollbar_position (emfv->list, position);
+	
+	emfb->priv->list_scrolled_id = g_signal_connect (emfv->list, "message_list_scrolled", G_CALLBACK (emfb_list_scrolled), emfb);
+	
+	emfb->priv->idle_scroll_id = 0;
+	
+	return FALSE;
+}
+
 /* TODO: This should probably be handled by message-list, by storing/queueing
    up the select operation if its busy rebuilding the message-list */
 static void
 emfb_list_built (MessageList *ml, EMFolderBrowser *emfb)
 {
 	EMFolderView *emfv = (EMFolderView *) emfb;
+	double position = 0.0f;
 	
 	g_signal_handler_disconnect (ml, emfb->priv->list_built_id);
 	emfb->priv->list_built_id = 0;
 	
 	if (emfv->list->cursor_uid == NULL) {
 		if (emfb->priv->select_uid) {
-			message_list_select_uid (ml, emfb->priv->select_uid);
+			em_folder_view_set_message(emfv, emfb->priv->select_uid, TRUE);
 			g_free (emfb->priv->select_uid);
 			emfb->priv->select_uid = NULL;
+			
+			/* change the default to the current position */
+			position = message_list_get_scrollbar_position (ml);
 		} else {
 			/* NOTE: not all users want this, so we need a preference for it perhaps? see bug #52887 */
 			/* FIXME: if the 1st message in the list is unread, this will actually select the second unread msg */
 			/*message_list_select (ml, MESSAGE_LIST_SELECT_NEXT, 0, CAMEL_MESSAGE_SEEN, TRUE);*/
 		}
 	}
-}
-
-
-static void
-emfb_list_display_view (GalViewInstance *instance, GalView *view, EMFolderBrowser *emfb)
-{
-	if (GAL_IS_VIEW_ETABLE (view))
-		gal_view_etable_attach_tree (GAL_VIEW_ETABLE (view), emfb->view.list->tree);
-}
-
-static void
-collection_init (void)
-{
-	ETableSpecification *spec;
-	GalViewFactory *factory;
-	const char *evolution_dir;
-	char *dir;
 	
-	if (collection != NULL)
-		return;
+	emfb->priv->default_scroll_position = position;
 	
-	collection = gal_view_collection_new ();
-	
-	gal_view_collection_set_title (collection, _("Mail"));
-	
-	evolution_dir = mail_component_peek_base_directory (mail_component_peek ());
-	dir = g_build_filename (evolution_dir, "mail", "views", NULL);
-	gal_view_collection_set_storage_directories (collection, EVOLUTION_GALVIEWSDIR "/mail/", dir);
-	g_free (dir);
-	
-	spec = e_table_specification_new ();
-	e_table_specification_load_from_file (spec, EVOLUTION_ETSPECDIR "/message-list.etspec");
-	
-	factory = gal_view_factory_etable_new (spec);
-	g_object_unref (spec);
-	gal_view_collection_add_factory (collection, factory);
-	g_object_unref (factory);
-	
-	gal_view_collection_load (collection);
-}
-
-static void
-emfb_create_view_instance (EMFolderBrowser *emfb, CamelFolder *folder, const char *uri)
-{
-	struct _EMFolderBrowserPrivate *priv = emfb->priv;
-	gboolean outgoing;
-	char *id;
-	
-	collection_init ();
-	
-	if (priv->view_instance) {
-		g_object_unref (priv->view_instance);
-		priv->view_instance = NULL;
-	}
-	
-	if (folder == NULL) {
-		folder = emfb->view.folder;
-		uri = emfb->view.folder_uri;
-	}
-	
-	outgoing = em_utils_folder_is_drafts (folder, uri)
-		|| em_utils_folder_is_sent (folder, uri)
-		|| em_utils_folder_is_outbox (folder, uri);
-	
-	/* TODO: should this go through mail-config api? */
-	id = mail_config_folder_to_safe_url (folder);
-	priv->view_instance = gal_view_instance_new (collection, id);
-	g_free (id);
-	
-	if (outgoing)
-		gal_view_instance_set_default_view (priv->view_instance, "As_Sent_Folder");
-	
-	gal_view_instance_load (priv->view_instance);
-	
-	if (!gal_view_instance_exists (priv->view_instance)) {
-		struct stat st;
-		char *path;
-		
-		path = mail_config_folder_to_cachename (folder, "et-header-");
-		if (path && stat (path, &st) == 0 && st.st_size > 0 && S_ISREG (st.st_mode)) {
-			ETableSpecification *spec;
-			ETableState *state;
-			GalView *view;
-			
-			spec = e_table_specification_new ();
-			e_table_specification_load_from_file (spec, EVOLUTION_ETSPECDIR "/message-list.etspec");
-			view = gal_view_etable_new (spec, "");
-			g_object_unref (spec);
-			
-			state = e_table_state_new ();
-			e_table_state_load_from_file (state, path);
-			gal_view_etable_set_state (GAL_VIEW_ETABLE (view), state);
-			g_object_unref (state);
-			
-			gal_view_instance_set_custom_view (priv->view_instance, view);
-			g_object_unref (view);
-		}
-		
-		g_free (path);
-	}
-	
-	if (priv->view_menus)
-		gal_view_menus_set_instance (priv->view_menus, priv->view_instance);
-	
-	g_signal_connect (priv->view_instance, "display_view", G_CALLBACK (emfb_list_display_view), emfb);	
-	emfb_list_display_view (priv->view_instance, gal_view_instance_get_current_view (priv->view_instance), emfb);
+	/* FIXME: this is a gross workaround for an etable bug that I can't fix - bug #55303 */
+	/* this needs to be a lower priority than anything in e-table-item/e-canvas, since
+	 * e_canvas_item_region_show_relay() uses a timeout, we have to use a timeout of the
+	 * same interval but a lower priority. */
+	emfb->priv->idle_scroll_id = g_timeout_add_full (G_PRIORITY_LOW, 250, (GSourceFunc) scroll_idle_cb, emfb, NULL);
 }
 
 static void
@@ -918,7 +862,19 @@ emfb_set_folder(EMFolderView *emfv, CamelFolder *folder, const char *uri)
 {
 	EMFolderBrowser *emfb = (EMFolderBrowser *) emfv;
 	struct _EMFolderBrowserPrivate *p = emfb->priv;
-
+	
+	message_list_freeze(emfv->list);
+	
+	if (emfb->priv->list_scrolled_id) {
+		g_signal_handler_disconnect (emfv->list, emfb->priv->list_scrolled_id);
+		emfb->priv->list_scrolled_id = 0;
+	}
+	
+	if (emfb->priv->idle_scroll_id) {
+		g_source_remove (emfb->priv->idle_scroll_id);
+		emfb->priv->idle_scroll_id = 0;
+	}
+	
 	emfb_parent->set_folder(emfv, folder, uri);
 	
 	/* This is required since we get activated the first time
@@ -948,6 +904,11 @@ emfb_set_folder(EMFolderView *emfv, CamelFolder *folder, const char *uri)
 		if (emfv->uic)
 			bonobo_ui_component_set_prop(emfv->uic, "/commands/ViewThreaded", "state", state?"1":"0", NULL);
 		
+		if (emfv->uic) {
+			state = (folder->folder_flags & CAMEL_FOLDER_IS_TRASH) == 0;
+			bonobo_ui_component_set_prop(emfv->uic, "/commands/HideDeleted", "sensitive", state?"1":"0", NULL);
+		}
+
 		sstate = camel_object_meta_get(folder, "evolution:search_state");
 		g_object_set(emfb->search, "state", sstate, NULL);
 		g_free(sstate);
@@ -957,62 +918,23 @@ emfb_set_folder(EMFolderView *emfv, CamelFolder *folder, const char *uri)
 		message_list_set_search(emfb->view.list, sstate);
 		g_free(sstate);
 
-		if ((sstate = camel_object_meta_get (folder, "evolution:selected_uid")))
+		if ((sstate = camel_object_meta_get (folder, "evolution:selected_uid"))) {
 			emfb->priv->select_uid = sstate;
-		else {
+		} else {
 			g_free(p->select_uid);
 			p->select_uid = NULL;
 		}
 		
 		if (emfv->list->cursor_uid == NULL && emfb->priv->list_built_id == 0)
-			emfb->priv->list_built_id = g_signal_connect(emfv->list, "message_list_built", G_CALLBACK (emfb_list_built), emfv);
-		
-		/*emfb_create_view_instance (emfb, folder, uri);*/
-		if (emfv->uic)
-			emfb_create_view_menus (emfb, emfv->uic);
-	}
-}
-
-/* TODO: All this mess should sit directly on MessageList, but it would
-   need to become BonoboUIComponent aware ... */
-
-static void
-emfb_create_view_menus(EMFolderBrowser *emfb, BonoboUIComponent *uic)
-{
-	struct _EMFolderBrowserPrivate *p = emfb->priv;
-
-	if (p->view_instance) {
-		g_object_unref(p->view_instance);
-		p->view_instance = NULL;
+			p->list_built_id = g_signal_connect(emfv->list, "message_list_built", G_CALLBACK (emfb_list_built), emfv);
 	}
 
-	if (p->view_menus) {
-		g_object_unref(p->view_menus);
-		p->view_menus = NULL;
-	}
-
-	/*g_assert(p->view_instance == NULL);
-	  g_assert(p->view_menus == NULL);*/
-	
-	collection_init ();
-	
-	emfb_create_view_instance (emfb, emfb->view.folder, emfb->view.folder_uri);
-	p->view_menus = gal_view_menus_new (p->view_instance);
-	gal_view_menus_apply(p->view_menus, uic, NULL);
-	
-	/* Due to CORBA reentrancy, the view could be gone now. */
-	if (p->view_instance == NULL)
-		return;
-	
-	g_signal_connect(p->view_instance, "display_view", G_CALLBACK(emfb_list_display_view), emfb);	
-	emfb_list_display_view(p->view_instance, gal_view_instance_get_current_view(p->view_instance), emfb);
+	message_list_thaw(emfv->list);
 }
 
 static void
 emfb_activate(EMFolderView *emfv, BonoboUIComponent *uic, int act)
 {
-	struct _EMFolderBrowserPrivate *p = ((EMFolderBrowser *)emfv)->priv;
-
 	if (act) {
 		GConfClient *gconf;
 		gboolean state;
@@ -1055,12 +977,14 @@ emfb_activate(EMFolderView *emfv, BonoboUIComponent *uic, int act)
 
 		/* HideDeleted */
 		state = !gconf_client_get_bool(gconf, "/apps/evolution/mail/display/show_deleted", NULL);
+		if (emfv->folder && (emfv->folder->folder_flags & CAMEL_FOLDER_IS_TRASH)) {
+			state = FALSE;
+			bonobo_ui_component_set_prop(uic, "/commands/HideDeleted", "sensitive", "0", NULL);
+		} else
+			bonobo_ui_component_set_prop(uic, "/commands/HideDeleted", "sensitive", "1", NULL);
 		bonobo_ui_component_set_prop(uic, "/commands/HideDeleted", "state", state ? "1" : "0", NULL);
 		bonobo_ui_component_add_listener(uic, "HideDeleted", emfb_hide_deleted, emfv);
-		if (!(emfv->folder && (emfv->folder->folder_flags & CAMEL_FOLDER_IS_TRASH)))
-			message_list_set_hidedeleted (emfv->list, state);
-		else
-			bonobo_ui_component_set_prop(uic, "/commands/HideDeleted", "sensitive", state?"1":"0", NULL);
+		em_folder_view_set_hide_deleted(emfv, state); /* <- not sure if this optimal, but it'll do */
 
 		/* FIXME: If we have no folder, we can't do a few of the lookups we need,
 		   perhaps we should postpone till we can */
@@ -1085,25 +1009,12 @@ emfb_activate(EMFolderView *emfv, BonoboUIComponent *uic, int act)
 
 		if (((EMFolderBrowser *)emfv)->search)
 			e_search_bar_set_ui_component((ESearchBar *)((EMFolderBrowser *)emfv)->search, uic);
-
-		if (emfv->folder)
-			emfb_create_view_menus((EMFolderBrowser *)emfv, uic);
 	} else {
 		const BonoboUIVerb *v;
 		
 		for (v = &emfb_verbs[0]; v->cname; v++)
 			bonobo_ui_component_remove_verb(uic, v->cname);
 
-		if (p->view_instance) {
-			g_object_unref(p->view_instance);
-			p->view_instance = NULL;
-		}
-
-		if (p->view_menus) {
-			g_object_unref(p->view_menus);
-			p->view_menus = NULL;
-		}
-
 		if (((EMFolderBrowser *)emfv)->search)
 			e_search_bar_set_ui_component((ESearchBar *)((EMFolderBrowser *)emfv)->search, NULL);
 
