@@ -57,8 +57,8 @@
 #include "em-marshal.h"
 #include "em-folder-tree-model.h"
 
-#define u(x) x			/* unread count debug */
-#define d(x) x
+#define u(x)			/* unread count debug */
+#define d(x)
 
 static GType col_types[] = {
 	G_TYPE_STRING,   /* display name */
@@ -184,6 +184,7 @@ sort_cb (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_data
 	char *aname, *bname;
 	CamelStore *store;
 	gboolean is_store;
+	int rv = -2;
 	
 	gtk_tree_model_get (model, a, COL_BOOL_IS_STORE, &is_store,
 			    COL_POINTER_CAMEL_STORE, &store,
@@ -193,34 +194,40 @@ sort_cb (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_data
 	if (is_store) {
 		/* On This Computer is always first and VFolders is always last */
 		if (!strcmp (aname, _("On This Computer")))
-			return -1;
-		if (!strcmp (bname, _("On This Computer")))
-			return 1;
-		if (!strcmp (aname, _("VFolders")))
-			return 1;
-		if (!strcmp (bname, _("VFolders")))
-			return -1;
+			rv = -1;
+		else if (!strcmp (bname, _("On This Computer")))
+			rv = 1;
+		else if (!strcmp (aname, _("VFolders")))
+			rv = 1;
+		else if (!strcmp (bname, _("VFolders")))
+			rv = -1;
 	} else if (store == vfolder_store) {
 		/* UNMATCHED is always last */
 		if (aname && !strcmp (aname, _("UNMATCHED")))
-			return 1;
-		if (bname && !strcmp (bname, _("UNMATCHED")))
-			return -1;
+			rv = 1;
+		else if (bname && !strcmp (bname, _("UNMATCHED")))
+			rv = -1;
 	} else {
 		/* Inbox is always first */
 		if (aname && (!strcmp (aname, "INBOX") || !strcmp (aname, _("Inbox"))))
-			return -1;
-		if (bname && (!strcmp (bname, "INBOX") || !strcmp (bname, _("Inbox"))))
-			return 1;
+			rv = -1;
+		else if (bname && (!strcmp (bname, "INBOX") || !strcmp (bname, _("Inbox"))))
+			rv = 1;
 	}
 	
 	if (aname == NULL) {
 		if (bname == NULL)
-			return 0;
+			rv = 0;
 	} else if (bname == NULL)
-		return 1;
+		rv = 1;
 	
-	return g_utf8_collate (aname, bname);
+	if (rv == -2)
+		rv = g_utf8_collate (aname, bname);
+	
+	g_free (aname);
+	g_free (bname);
+	
+	return rv;
 }
 
 static void
@@ -238,7 +245,7 @@ em_folder_tree_model_init (EMFolderTreeModel *model)
 }
 
 static void
-path_hash_free (gpointer key, gpointer value, gpointer user_data)
+full_hash_free (gpointer key, gpointer value, gpointer user_data)
 {
 	g_free (key);
 	gtk_tree_row_reference_free (value);
@@ -256,7 +263,7 @@ store_info_free (struct _EMFolderTreeModelStoreInfo *si)
 	g_free (si->display_name);
 	camel_object_unref (si->store);
 	gtk_tree_row_reference_free (si->row);
-	g_hash_table_foreach (si->path_hash, path_hash_free, NULL);
+	g_hash_table_foreach (si->full_hash, full_hash_free, NULL);
 	g_free (si);
 }
 
@@ -281,8 +288,8 @@ em_folder_tree_model_finalize (GObject *obj)
 	EMFolderTreeModel *model = (EMFolderTreeModel *) obj;
 	
 	g_free (model->filename);
-	if (model->expanded)
-		xmlFreeDoc (model->expanded);
+	if (model->state)
+		xmlFreeDoc (model->state);
 	
 	g_hash_table_foreach (model->store_hash, store_hash_free, NULL);
 	g_hash_table_destroy (model->store_hash);
@@ -317,16 +324,16 @@ em_folder_tree_model_load_state (EMFolderTreeModel *model, const char *filename)
 	xmlNodePtr root, node;
 	struct stat st;
 	
-	if (model->expanded)
-		xmlFreeDoc (model->expanded);
+	if (model->state)
+		xmlFreeDoc (model->state);
 	
-	if (stat (filename, &st) == 0 && (model->expanded = xmlParseFile (filename)))
+	if (stat (filename, &st) == 0 && (model->state = xmlParseFile (filename)))
 		return;
 	
 	/* setup some defaults - expand "Local Folders" and "VFolders" */
-	model->expanded = xmlNewDoc ("1.0");
-	root = xmlNewDocNode (model->expanded, NULL, "tree-state", NULL);
-	xmlDocSetRootElement (model->expanded, root);
+	model->state = xmlNewDoc ("1.0");
+	root = xmlNewDocNode (model->state, NULL, "tree-state", NULL);
+	xmlDocSetRootElement (model->state, root);
 	
 	node = xmlNewChild (root, NULL, "node", NULL);
 	xmlSetProp (node, "name", "local");
@@ -430,7 +437,7 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model, GtkTreeIter *ite
 	gtk_tree_path_free (path);
 	
 	g_hash_table_insert (model->uri_hash, g_strdup (fi->uri), uri_row);
-	g_hash_table_insert (si->path_hash, g_strdup (fi->path), path_row);
+	g_hash_table_insert (si->full_hash, g_strdup (fi->full_name), path_row);
 	
 	/* HACK: if we have the folder, and its the outbox folder, we need the total count, not unread */
 	/* This is duplicated in mail-folder-cache too, should perhaps be functionised */
@@ -454,7 +461,7 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model, GtkTreeIter *ite
 	gtk_tree_store_set ((GtkTreeStore *) model, iter,
 			    COL_STRING_DISPLAY_NAME, fi->name,
 			    COL_POINTER_CAMEL_STORE, si->store,
-			    COL_STRING_FOLDER_PATH, fi->path,
+			    COL_STRING_FULL_NAME, fi->full_name,
 			    COL_STRING_URI, fi->uri,
 			    COL_UINT_UNREAD, unread,
 			    COL_UINT_FLAGS, fi->flags,
@@ -468,7 +475,7 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model, GtkTreeIter *ite
 		gtk_tree_store_set ((GtkTreeStore *) model, &sub,
 				    COL_STRING_DISPLAY_NAME, _("Loading..."),
 				    COL_POINTER_CAMEL_STORE, NULL,
-				    COL_STRING_FOLDER_PATH, NULL,
+				    COL_STRING_FULL_NAME, NULL,
 				    COL_BOOL_LOAD_SUBDIRS, FALSE,
 				    COL_BOOL_IS_STORE, FALSE,
 				    COL_STRING_URI, NULL,
@@ -515,26 +522,25 @@ folder_subscribed (CamelStore *store, CamelFolderInfo *fi, EMFolderTreeModel *mo
 	GtkTreeIter parent, iter;
 	GtkTreePath *path;
 	gboolean load;
-	char *dirname;
-	
+	char *dirname, *p;
+
 	if (!(si = g_hash_table_lookup (model->store_hash, store)))
 		goto done;
 	
 	/* make sure we don't already know about it? */
-	if (g_hash_table_lookup (si->path_hash, fi->path))
+	if (g_hash_table_lookup (si->full_hash, fi->full_name))
 		goto done;
 	
 	/* get our parent folder's path */
-	if (!(dirname = g_path_get_dirname (fi->path)))
-		goto done;
-	
-	if (!strcmp (dirname, "/")) {
+	dirname = alloca(strlen(fi->full_name)+1);
+	strcpy(dirname, fi->full_name);
+	p = strrchr(dirname, '/');
+	if (p == NULL) {
 		/* user subscribed to a toplevel folder */
 		row = si->row;
-		g_free (dirname);
 	} else {
-		row = g_hash_table_lookup (si->path_hash, dirname);
-		g_free (dirname);
+		*p = 0;
+		row = g_hash_table_lookup (si->full_hash, dirname);
 		
 		/* if row is NULL, don't bother adding to the tree,
 		 * when the user expands enough nodes - it will be
@@ -560,8 +566,8 @@ folder_subscribed (CamelStore *store, CamelFolderInfo *fi, EMFolderTreeModel *mo
 	gtk_tree_store_append ((GtkTreeStore *) model, &iter, &parent);
 	
 	em_folder_tree_model_set_folder_info (model, &iter, si, fi, TRUE);
-	
-	g_signal_emit (model, signals[FOLDER_ADDED], 0, fi->path, fi->uri);
+
+	g_signal_emit (model, signals[FOLDER_ADDED], 0, fi->full_name, fi->uri);
 	
  done:
 	
@@ -590,7 +596,7 @@ folder_unsubscribed (CamelStore *store, CamelFolderInfo *fi, EMFolderTreeModel *
 	if (!(si = g_hash_table_lookup (model->store_hash, store)))
 		goto done;
 	
-	if (!(row = g_hash_table_lookup (si->path_hash, fi->path)))
+	if (!(row = g_hash_table_lookup (si->full_hash, fi->full_name)))
 		goto done;
 	
 	path = gtk_tree_row_reference_get_path (row);
@@ -621,14 +627,14 @@ static void
 folder_created_cb (CamelStore *store, void *event_data, EMFolderTreeModel *model)
 {
 	CamelFolderInfo *fi;
-	
+
 	/* we only want created events to do more work if we don't support subscriptions */
 	if (camel_store_supports_subscriptions (store))
 		return;
 	
 	camel_object_ref (store);
 	fi = camel_folder_info_clone (event_data);
-	mail_async_event_emit (mail_async_event, MAIL_ASYNC_GUI, (MailAsyncFunc) folder_subscribed_cb, store, fi, model);
+	mail_async_event_emit (mail_async_event, MAIL_ASYNC_GUI, (MailAsyncFunc) folder_subscribed, store, fi, model);
 }
 
 static void
@@ -653,16 +659,12 @@ folder_renamed (CamelStore *store, CamelRenameInfo *info, EMFolderTreeModel *mod
 	GtkTreeIter root, iter;
 	GtkTreePath *path;
 	char *parent, *p;
-	
+
 	if (!(si = g_hash_table_lookup (model->store_hash, store)))
 		goto done;
 	
-	parent = g_strdup_printf ("/%s", info->old_base);
-	if (!(row = g_hash_table_lookup (si->path_hash, parent))) {
-		g_free (parent);
+	if (!(row = g_hash_table_lookup (si->full_hash, info->old_base)))
 		goto done;
-	}
-	g_free (parent);
 	
 	path = gtk_tree_row_reference_get_path (row);
 	if (!(gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, path))) {
@@ -672,15 +674,15 @@ folder_renamed (CamelStore *store, CamelRenameInfo *info, EMFolderTreeModel *mod
 	
 	em_folder_tree_model_remove_folders (model, si, &iter);
 	
-	parent = g_strdup (info->new->path);
+	parent = g_strdup(info->new->full_name);
 	p = strrchr(parent, '/');
-	g_assert(p);
-	*p = 0;
-	if (parent == p) {
+	if (p)
+		*p = 0;
+	if (p == NULL || parent == p) {
 		/* renamed to a toplevel folder on the store */
 		path = gtk_tree_row_reference_get_path (si->row);
 	} else {
-		if (!(row = g_hash_table_lookup (si->path_hash, parent))) {
+		if (!(row = g_hash_table_lookup (si->full_hash, parent))) {
 			/* NOTE: this should never happen, but I
 			 * suppose if it does in reality, we can add
 			 * code here to add the missing nodes to the
@@ -753,7 +755,7 @@ em_folder_tree_model_add_store (EMFolderTreeModel *model, CamelStore *store, con
 	gtk_tree_store_set ((GtkTreeStore *) model, &iter,
 			    COL_STRING_DISPLAY_NAME, display_name,
 			    COL_POINTER_CAMEL_STORE, store,
-			    COL_STRING_FOLDER_PATH, "/",
+			    COL_STRING_FULL_NAME, NULL,
 			    COL_BOOL_LOAD_SUBDIRS, TRUE,
 			    COL_BOOL_IS_STORE, TRUE,
 			    COL_STRING_URI, uri, -1);
@@ -767,7 +769,7 @@ em_folder_tree_model_add_store (EMFolderTreeModel *model, CamelStore *store, con
 	si->store = store;
 	si->account = account;
 	si->row = row;
-	si->path_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	si->full_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	g_hash_table_insert (model->store_hash, store, si);
 	g_hash_table_insert (model->account_hash, account, si);
 	
@@ -777,7 +779,7 @@ em_folder_tree_model_add_store (EMFolderTreeModel *model, CamelStore *store, con
 	gtk_tree_store_set ((GtkTreeStore *) model, &iter,
 			    COL_STRING_DISPLAY_NAME, _("Loading..."),
 			    COL_POINTER_CAMEL_STORE, NULL,
-			    COL_STRING_FOLDER_PATH, NULL,
+			    COL_STRING_FULL_NAME, NULL,
 			    COL_BOOL_LOAD_SUBDIRS, FALSE,
 			    COL_BOOL_IS_STORE, FALSE,
 			    COL_STRING_URI, NULL,
@@ -835,7 +837,7 @@ void
 em_folder_tree_model_remove_folders (EMFolderTreeModel *model, struct _EMFolderTreeModelStoreInfo *si, GtkTreeIter *toplevel)
 {
 	GtkTreeRowReference *row;
-	char *uri, *folder_path;
+	char *uri, *full_name;
 	gboolean is_store, go;
 	GtkTreeIter iter;
 	
@@ -850,11 +852,11 @@ em_folder_tree_model_remove_folders (EMFolderTreeModel *model, struct _EMFolderT
 	}
 	
 	gtk_tree_model_get ((GtkTreeModel *) model, toplevel, COL_STRING_URI, &uri,
-			    COL_STRING_FOLDER_PATH, &folder_path,
+			    COL_STRING_FULL_NAME, &full_name,
 			    COL_BOOL_IS_STORE, &is_store, -1);
 	
-	if (folder_path && (row = g_hash_table_lookup (si->path_hash, folder_path))) {
-		g_hash_table_remove (si->path_hash, folder_path);
+	if (full_name && (row = g_hash_table_lookup (si->full_hash, full_name))) {
+		g_hash_table_remove (si->full_hash, full_name);
 		gtk_tree_row_reference_free (row);
 	}
 	
@@ -864,6 +866,9 @@ em_folder_tree_model_remove_folders (EMFolderTreeModel *model, struct _EMFolderT
 	
 	if (is_store)
 		em_folder_tree_model_remove_store_info (model, si->store);
+	
+	g_free (full_name);
+	g_free (uri);
 }
 
 
@@ -920,7 +925,7 @@ em_folder_tree_model_get_expanded (EMFolderTreeModel *model, const char *key)
 	const char *name;
 	char *buf, *p;
 	
-	node = model->expanded ? model->expanded->children : NULL;
+	node = model->state ? model->state->children : NULL;
 	if (!node || strcmp (node->name, "tree-state") != 0)
 		return FALSE;
 	
@@ -959,14 +964,14 @@ em_folder_tree_model_set_expanded (EMFolderTreeModel *model, const char *key, gb
 	const char *name;
 	char *buf, *p;
 	
-	if (model->expanded == NULL)
-		model->expanded = xmlNewDoc ("1.0");
+	if (model->state == NULL)
+		model->state = xmlNewDoc ("1.0");
 	
-	if (!model->expanded->children) {
-		node = xmlNewDocNode (model->expanded, NULL, "tree-state", NULL);
-		xmlDocSetRootElement (model->expanded, node);
+	if (!model->state->children) {
+		node = xmlNewDocNode (model->state, NULL, "tree-state", NULL);
+		xmlDocSetRootElement (model->state, node);
 	} else {
-		node = model->expanded->children;
+		node = model->state->children;
 	}
 	
 	name = buf = g_alloca (strlen (key) + 1);
@@ -998,11 +1003,11 @@ em_folder_tree_model_set_expanded (EMFolderTreeModel *model, const char *key, gb
 }
 
 void
-em_folder_tree_model_save_expanded (EMFolderTreeModel *model)
+em_folder_tree_model_save_state (EMFolderTreeModel *model)
 {
 	char *dirname;
 	
-	if (model->expanded == NULL)
+	if (model->state == NULL)
 		return;
 	
 	dirname = g_path_get_dirname (model->filename);
@@ -1013,7 +1018,7 @@ em_folder_tree_model_save_expanded (EMFolderTreeModel *model)
 	
 	g_free (dirname);
 	
-	e_xml_save_file (model->filename, model->expanded);
+	e_xml_save_file (model->filename, model->state);
 }
 
 
@@ -1053,7 +1058,7 @@ em_folder_tree_model_expand_foreach (EMFolderTreeModel *model, EMFTModelExpandFu
 {
 	xmlNodePtr root;
 	
-	root = model->expanded ? model->expanded->children : NULL;
+	root = model->state ? model->state->children : NULL;
 	if (!root || !root->children || strcmp (root->name, "tree-state") != 0)
 		return;
 	
@@ -1061,7 +1066,7 @@ em_folder_tree_model_expand_foreach (EMFolderTreeModel *model, EMFTModelExpandFu
 }
 
 void
-em_folder_tree_model_set_unread_count (EMFolderTreeModel *model, CamelStore *store, const char *path, int unread)
+em_folder_tree_model_set_unread_count (EMFolderTreeModel *model, CamelStore *store, const char *full, int unread)
 {
 	struct _EMFolderTreeModelStoreInfo *si;
 	GtkTreeRowReference *row;
@@ -1070,9 +1075,9 @@ em_folder_tree_model_set_unread_count (EMFolderTreeModel *model, CamelStore *sto
 	
 	g_return_if_fail (EM_IS_FOLDER_TREE_MODEL (model));
 	g_return_if_fail (CAMEL_IS_STORE (store));
-	g_return_if_fail (path != NULL);
+	g_return_if_fail (full != NULL);
 
-	u(printf("set unread count %p '%s' %d\n", store, path, unread));
+	u(printf("set unread count %p '%s' %d\n", store, full, unread));
 
 	if (unread < 0)
 		unread = 0;
@@ -1082,7 +1087,7 @@ em_folder_tree_model_set_unread_count (EMFolderTreeModel *model, CamelStore *sto
 		return;
 	}
 	
-	if (!(row = g_hash_table_lookup (si->path_hash, path))) {
+	if (!(row = g_hash_table_lookup (si->full_hash, full))) {
 		u(printf("  can't find row\n"));
 		return;
 	}
@@ -1097,3 +1102,60 @@ em_folder_tree_model_set_unread_count (EMFolderTreeModel *model, CamelStore *sto
 	
 	gtk_tree_store_set ((GtkTreeStore *) model, &iter, COL_UINT_UNREAD, unread, -1);
 }
+
+
+char *
+em_folder_tree_model_get_selected (EMFolderTreeModel *model)
+{
+	xmlNodePtr node;
+	char *buf, *uri;
+	
+	node = model->state ? model->state->children : NULL;
+	if (!node || strcmp (node->name, "tree-state") != 0)
+		return FALSE;
+	
+	node = node->children;
+	while (node != NULL) {
+		if (!strcmp (node->name, "selected"))
+			break;
+		node = node->next;
+	}
+	
+	if (node == NULL)
+		return NULL;
+	
+	buf = xmlGetProp (node, "uri");
+	uri = g_strdup (buf);
+	xmlFree (buf);
+	
+	return uri;
+}
+
+
+void
+em_folder_tree_model_set_selected (EMFolderTreeModel *model, const char *uri)
+{
+	xmlNodePtr root, node;
+	
+	if (model->state == NULL)
+		model->state = xmlNewDoc ("1.0");
+	
+	if (!model->state->children) {
+		root = xmlNewDocNode (model->state, NULL, "tree-state", NULL);
+		xmlDocSetRootElement (model->state, node);
+	} else {
+		root = model->state->children;
+	}
+	
+	node = root->children;
+	while (node != NULL) {
+		if (!strcmp (node->name, "selected"))
+			break;
+		node = node->next;
+	}
+	
+	if (node == NULL)
+		node = xmlNewChild (root, NULL, "selected", NULL);
+	
+	xmlSetProp (node, "uri", uri);
+}
