@@ -40,9 +40,14 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkarrow.h>
-
+#include <gtk/gtklabel.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtktable.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkdnd.h>
 
 #include <glade/glade.h>
 
@@ -66,6 +71,8 @@
 #include <camel/camel-internet-address.h>
 #include <camel/camel-mime-message.h>
 #include <camel/camel-cipher-context.h>
+#include <camel/camel-folder.h>
+#include <camel/camel-string-utils.h>
 
 /* should this be in e-util rather than gal? */
 #include <gal/util/e-util.h>
@@ -73,6 +80,7 @@
 #include <e-util/e-msgport.h>
 #include <e-util/e-gui-utils.h>
 #include <e-util/e-dialog-utils.h>
+#include <e-util/e-icon-factory.h>
 
 #ifdef HAVE_NSS
 #include "certificate-viewer.h"
@@ -304,7 +312,7 @@ efhd_class_init(GObjectClass *klass)
 			     G_SIGNAL_RUN_LAST,
 			     G_STRUCT_OFFSET(EMFormatHTMLDisplayClass, on_url),
 			     NULL, NULL,
-			     gtk_marshal_VOID__STRING,
+			     g_cclosure_marshal_VOID__STRING,
 			     G_TYPE_NONE, 1,
 			     G_TYPE_STRING);
 
@@ -415,9 +423,6 @@ efhd_update_search(EMFormatHTMLDisplay *efhd)
 	GSList *words = NULL;
 	int flags = 0;
 
-	if (p->search_text == NULL)
-		return;
-
 	if (!gtk_toggle_button_get_active((GtkToggleButton *)p->search_case_check))
 		flags = EM_FORMAT_HTML_DISPLAY_SEARCH_ICASE | EM_FORMAT_HTML_DISPLAY_SEARCH_PRIMARY;
 	else
@@ -458,6 +463,7 @@ efhd_search_response(GtkWidget *w, int button, EMFormatHTMLDisplay *efhd)
 		p->search_text = NULL;
 		gtk_widget_destroy((GtkWidget *)p->search_dialog);
 		p->search_dialog = NULL;
+		em_format_html_display_set_search(efhd, EM_FORMAT_HTML_DISPLAY_SEARCH_PRIMARY, NULL);
 	}
 }
 
@@ -494,7 +500,7 @@ em_format_html_display_search(EMFormatHTMLDisplay *efhd)
 		return;
 	}
 
-	xml = glade_xml_new (EVOLUTION_GLADEDIR "/mail-search.glade", "search_message_dialog", NULL);
+	xml = glade_xml_new (EVOLUTION_GLADEDIR "/mail-dialogs.glade", "search_message_dialog", NULL);
 	if (xml == NULL) {
 		g_warning("Cannot open search dialog glade file");
 		/* ?? */
@@ -516,7 +522,6 @@ em_format_html_display_search(EMFormatHTMLDisplay *efhd)
 	g_signal_connect(p->search_entry, "activate", G_CALLBACK(efhd_search_entry_activate), efhd);
 	g_signal_connect(p->search_case_check, "toggled", G_CALLBACK(efhd_search_case_toggled), efhd);
 	g_signal_connect(p->search_dialog, "response", G_CALLBACK(efhd_search_response), efhd);
-	e_dialog_set_transient_for((GtkWindow *)p->search_dialog, (GtkWidget *)efhd);
 	gtk_widget_show((GtkWidget *)p->search_dialog);
 }
 
@@ -576,31 +581,30 @@ efhd_html_button_press_event (GtkWidget *widget, GdkEventButton *event, EMFormat
 	const char *url;
 	gboolean res = FALSE;
 	gint offset;
+	EMFormatPURI *puri = NULL;
+	char *uri = NULL;
 
 	if (event->button != 3)
 		return FALSE;
 
 	e = ((GtkHTML *)widget)->engine;
 	obj = html_engine_get_object_at(e, event->x, event->y, &offset, FALSE);
-	if (obj == NULL)
-		return FALSE;
 
 	d(printf("popup button pressed\n"));
 
-	if ( (url = html_object_get_src(obj)) != NULL
-	     || (url = html_object_get_url(obj, offset)) != NULL) {
-		EMFormatPURI *puri;
-		char *uri;
-
+	if ( obj != NULL
+	     && ((url = html_object_get_src(obj)) != NULL
+		 || (url = html_object_get_url(obj, offset)) != NULL)) {
 		uri = gtk_html_get_url_object_relative((GtkHTML *)widget, obj, url);
 		puri = em_format_find_puri((EMFormat *)efhd, uri);
 
 		d(printf("poup event, uri = '%s' part = '%p'\n", uri, puri?puri->part:NULL));
-
-		g_signal_emit((GtkObject *)efhd, efhd_signals[EFHD_POPUP_EVENT], 0, event, uri, puri?puri->part:NULL, &res);
-		g_free(uri);
 	}
 
+	g_signal_emit((GtkObject *)efhd, efhd_signals[EFHD_POPUP_EVENT], 0, event, uri, puri?puri->part:NULL, &res);
+
+	g_free(uri);
+
 	return res;
 }
 
@@ -629,25 +633,28 @@ efhd_complete(EMFormat *emf)
 
 /* ********************************************************************** */
 
-/* TODO: rename some of this stuff, it isn't 'smime' specific */
 /* TODO: move the dialogue elsehwere */
 /* FIXME: also in em-format-html.c */
 static const struct {
 	const char *icon, *shortdesc, *description;
 } smime_sign_table[4] = {
-	{ NULL, N_("Unsigned"), N_("This message is not signed. There is no guarantee that this message is authentic.") },
-	{ "pgp-signature-ok.png", N_("Valid signature"), N_("This message is signed and is valid meaning that it is very likely that this message is authentic.") },
-	{ "pgp-signature-bad.png", N_("Invalid signature"), N_("The signature of this message cannot be verified, it may have been altered in transit.") },
-	{ "pgp-signature-nokey.png", N_("Valid signature, cannot verify sender"), N_("This message is signed with a valid signature, but the sender of the message cannot be verified.") },
+	{ "stock_signature-bad", N_("Unsigned"), N_("This message is not signed. There is no guarantee that this message is authentic.") },
+	{ "stock_signature-ok", N_("Valid signature"), N_("This message is signed and is valid meaning that it is very likely that this message is authentic.") },
+	{ "stock_signature-bad", N_("Invalid signature"), N_("The signature of this message cannot be verified, it may have been altered in transit.") },
+	{ "stock_signature", N_("Valid signature, cannot verify sender"), N_("This message is signed with a valid signature, but the sender of the message cannot be verified.") },
 };
 
 static const struct {
 	const char *icon, *shortdesc, *description;
 } smime_encrypt_table[4] = {
-	{ NULL, N_("Unencrypted"), N_("This message is not encrypted.  Its content may be viewed in transit across the Internet.") },
-	{ "pgp-signature-ok.png", N_("Encrypted, weak"), N_("This message is encrypted, but with a weak encryption algorithm.  It would be difficult, but not impossible for an outsider to view the content of this message in a practical amount of time.") },
-	{ "pgp-signature-ok.png", N_("Encrypted"), N_("This message is encrypted.  It would be difficult for an outsider to view the content of this message.") },
-	{ "pgp-signature-ok.png", N_("Encrypted, strong"), N_("This message is encrypted, with a strong encryption algorithm.  It would be very difficult for an outsider to view the content of this message in a practical amount of time.") },
+	{ "stock_lock-broken", N_("Unencrypted"), N_("This message is not encrypted.  Its content may be viewed in transit across The Internet.") },
+	{ "stock_lock-ok", N_("Encrypted, weak"), N_("This message is encrypted, but with a weak encryption algorithm.  It would be difficult, but not impossible for an outsider to view the content of this message in a practical amount of time.") },
+	{ "stock_lock-ok", N_("Encrypted"), N_("This message is encrypted.  It would be difficult for an outsider to view the content of this message.") },
+	{ "stock_lock-ok", N_("Encrypted, strong"), N_("This message is encrypted, with a strong encryption algorithm.  It would be very difficult for an outsider to view the content of this message in a practical amount of time.") },
+};
+
+static const char *smime_sign_colour[4] = {
+	"", " bgcolor=\"#88bb88\"", " bgcolor=\"#bb8888\"", " bgcolor=\"#e8d122\""
 };
 
 struct _smime_pobject {
@@ -675,6 +682,12 @@ efhd_xpkcs7mime_info_response(GtkWidget *w, guint button, struct _smime_pobject
 	po->widget = NULL;
 }
 
+static void
+efhd_xpkcs7mime_viewcert_foad(GtkWidget *w, guint button, struct _smime_pobject *po)
+{
+	gtk_widget_destroy(w);
+}
+
 #ifdef HAVE_NSS
 static void
 efhd_xpkcs7mime_viewcert_clicked(GtkWidget *button, struct _smime_pobject *po)
@@ -692,6 +705,10 @@ efhd_xpkcs7mime_viewcert_clicked(GtkWidget *button, struct _smime_pobject *po)
 	if (ec != NULL) {
 		GtkWidget *w = certificate_viewer_show(ec);
 
+		/* oddly enough certificate_viewer_show doesn't ... */
+		gtk_widget_show(w);
+		g_signal_connect(w, "response", G_CALLBACK(efhd_xpkcs7mime_viewcert_foad), po);
+
 		if (w && po->widget)
 			gtk_window_set_transient_for((GtkWindow *)w, (GtkWindow *)po->widget);
 
@@ -725,7 +742,10 @@ efhd_xpkcs7mime_add_cert_table(GtkWidget *vbox, EDList *certlist, struct _smime_
 		
 		if (l) {
 			GtkWidget *w;
-
+#if defined(HAVE_NSS)
+			ECertDB *db = e_cert_db_peek();
+			ECert *ec = NULL;
+#endif
 			w = gtk_label_new(l);
 			gtk_misc_set_alignment((GtkMisc *)w, 0.0, 0.5);
 			g_free(la);
@@ -735,6 +755,16 @@ efhd_xpkcs7mime_add_cert_table(GtkWidget *vbox, EDList *certlist, struct _smime_
 			gtk_table_attach(table, w, 1, 2, n, n+1, 0, 0, 3, 3);
 			g_object_set_data((GObject *)w, "e-cert-info", info);
 			g_signal_connect(w, "clicked", G_CALLBACK(efhd_xpkcs7mime_viewcert_clicked), po);
+
+			if (info->email)
+				ec = e_cert_db_find_cert_by_email_address(db, info->email, NULL);
+			if (ec == NULL && info->name)
+				ec = e_cert_db_find_cert_by_nickname(db, info->name, NULL);
+
+			if (ec == NULL)
+				gtk_widget_set_sensitive(w, FALSE);
+			else
+				g_object_unref(ec);
 #else
 			w = gtk_label_new (_("This certificate is not viewable"));
 			gtk_table_attach(table, w, 1, 2, n, n+1, 0, 0, 3, 3);
@@ -752,7 +782,6 @@ static void
 efhd_xpkcs7mime_validity_clicked(GtkWidget *button, EMFormatHTMLPObject *pobject)
 {
 	struct _smime_pobject *po = (struct _smime_pobject *)pobject;
-	const char *alt_desc = NULL;
 	GladeXML *xml;
 	GtkWidget *vbox, *w;
 
@@ -760,7 +789,7 @@ efhd_xpkcs7mime_validity_clicked(GtkWidget *button, EMFormatHTMLPObject *pobject
 		/* FIXME: window raise? */
 		return;
 
-	xml = glade_xml_new(EVOLUTION_GLADEDIR "/mail-security.glade", "message_security_dialog", NULL);
+	xml = glade_xml_new(EVOLUTION_GLADEDIR "/mail-dialogs.glade", "message_security_dialog", NULL);
 	po->widget = glade_xml_get_widget(xml, "message_security_dialog");
 
 	vbox = glade_xml_get_widget(xml, "signature_vbox");
@@ -809,20 +838,17 @@ efhd_xpkcs7mime_button(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObje
 	GtkWidget *icon, *button;
 	GdkPixbuf *pixbuf;
 	struct _smime_pobject *po = (struct _smime_pobject *)pobject;
-	char *file;
 	const char *name;
 
 	/* FIXME: need to have it based on encryption and signing too */
-	name = smime_sign_table[po->valid->sign.status].icon;
-	
-	file = g_build_filename(EVOLUTION_ICONSDIR, name, NULL);
-	pixbuf = gdk_pixbuf_new_from_file(file, NULL);
-	g_free(file);
-	if (pixbuf == NULL)
-		return FALSE;
+	if (po->valid->sign.status != 0)
+		name = smime_sign_table[po->valid->sign.status].icon;
+	else
+		name = smime_encrypt_table[po->valid->encrypt.status].icon;
 
-	/* wtf isn't this just scaled on disk? */
-	icon = gtk_image_new_from_pixbuf(gdk_pixbuf_scale_simple(pixbuf, 24, 24, GDK_INTERP_BILINEAR));
+	pixbuf = e_icon_factory_get_icon (name, E_ICON_SIZE_LARGE_TOOLBAR);
+	
+	icon = gtk_image_new_from_pixbuf (pixbuf);
 	g_object_unref(pixbuf);
 	gtk_widget_show(icon);
 
@@ -847,16 +873,16 @@ efhd_format_secure(EMFormat *emf, CamelStream *stream, CamelMimePart *part, Came
 		|| valid->sign.status != CAMEL_CIPHER_VALIDITY_SIGN_NONE)) {
 		char *classid;
 		struct _smime_pobject *pobj;
-
-		camel_stream_printf(stream, "<table border=0 width=\"100%%\" cellpadding=3 cellspacing=0 bgcolor=%s><tr>",
-				    valid->sign.status == CAMEL_CIPHER_VALIDITY_SIGN_GOOD?"#88bb88":"#bb8888");
-
+		
+		camel_stream_printf (stream, "<table border=0 width=\"100%%\" cellpadding=3 cellspacing=0%s><tr>",
+				     smime_sign_colour[valid->sign.status]);
+		
 		classid = g_strdup_printf("smime:///em-format-html/%s/icon/signed", emf->part_id->str);
 		pobj = (struct _smime_pobject *)em_format_html_add_pobject((EMFormatHTML *)emf, sizeof(*pobj), classid, part, efhd_xpkcs7mime_button);
 		pobj->valid = camel_cipher_validity_clone(valid);
 		pobj->object.free = efhd_xpkcs7mime_free;
 		camel_stream_printf(stream, "<td valign=top><object classid=\"%s\"></object></td><td width=100%% valign=top>", classid);
-
+		g_free(classid);
 		if (valid->sign.status != CAMEL_CIPHER_VALIDITY_SIGN_NONE) {
 			camel_stream_printf(stream, "%s<br>", _(smime_sign_table[valid->sign.status].shortdesc));
 		}
@@ -892,7 +918,7 @@ efhd_bonobo_unknown(EMFormat *emf, CamelStream *stream, CamelMimePart *part, con
 
 	classid = g_strdup_printf("bonobo-unknown:///em-format-html-display/%s", emf->part_id->str);
 	em_format_html_add_pobject((EMFormatHTML *)emf, sizeof(EMFormatHTMLPObject), classid, part, efhd_bonobo_object);
-	camel_stream_printf(stream, "<object classid=\"%s\" type=\"%s\">\n", classid, info->mime_type);
+	camel_stream_printf(stream, "<object classid=\"%s\" type=\"%s\"></object><br>\n", classid, info->mime_type);
 	g_free(classid);
 }
 
@@ -939,10 +965,10 @@ efhd_write_image(EMFormat *emf, CamelStream *stream, EMFormatPURI *puri)
 static void efhd_format_prefix(EMFormat *emf, CamelStream *stream)
 {
 	const char *flag, *comp, *due;
-	CamelMimePart *iconpart;
 	time_t date;
 	char due_date[128];
 	struct tm due_tm;
+	char *iconpath;
 
 	if (emf->folder == NULL || emf->uid == NULL
 	    || (flag = camel_folder_get_message_user_tag(emf->folder, emf->uid, "follow-up")) == NULL
@@ -953,16 +979,21 @@ static void efhd_format_prefix(EMFormat *emf, CamelStream *stream)
 	camel_stream_printf(stream, "<table border=1 width=\"100%%\" cellspacing=2 cellpadding=2><tr>");
 
 	comp = camel_folder_get_message_user_tag(emf->folder, emf->uid, "completed-on");
-	iconpart = em_format_html_file_part((EMFormatHTML *)emf, "image/png", EVOLUTION_ICONSDIR,
-					    comp&&comp[0]?"flag-for-followup-done-16.png":"flag-for-followup-16.png");
-	if (iconpart) {
-		char *classid;
+	iconpath = e_icon_factory_get_icon_filename (comp && comp[0] ? "stock_flag-for-followup-done" : "stock_flag-for-followup", E_ICON_SIZE_MENU);
+	if (iconpath) {
+		CamelMimePart *iconpart;
 
-		classid = g_strdup_printf("icon:///em-format-html-display/%s/%s", emf->part_id->str, comp&&comp[0]?"comp":"uncomp");
-		camel_stream_printf(stream, "<td align=\"left\"><img src=\"%s\"></td>", classid);
-		(void)em_format_add_puri(emf, sizeof(EMFormatPURI), classid, iconpart, efhd_write_image);
-		g_free(classid);
-		camel_object_unref(iconpart);
+		iconpart = em_format_html_file_part((EMFormatHTML *)emf, "image/png", iconpath);
+		g_free (iconpath);
+		if (iconpart) {
+			char *classid;
+
+			classid = g_strdup_printf("icon:///em-format-html-display/%s/%s", emf->part_id->str, comp&&comp[0]?"comp":"uncomp");
+			camel_stream_printf(stream, "<td align=\"left\"><img src=\"%s\"></td>", classid);
+			(void)em_format_add_puri(emf, sizeof(EMFormatPURI), classid, iconpart, efhd_write_image);
+			g_free(classid);
+			camel_object_unref(iconpart);
+		}
 	}
 
 	camel_stream_printf(stream, "<td align=\"left\" width=\"100%%\">");
@@ -1086,11 +1117,15 @@ efhd_attachment_popup(GtkWidget *w, GdkEventButton *event, struct _attach_puri *
 	target->widget = w;
 
 	/* add our local menus */
-	efhd_menu_items[0].activate_data = info;
-	menus = g_slist_prepend(menus, &efhd_menu_items[0]);
-	item = &efhd_menu_items[info->shown?2:1];
-	item->activate_data = info;
-	menus = g_slist_prepend(menus, item);
+	if (info->handle) {
+		/* show/hide menus, only if we have an inline handler */
+		efhd_menu_items[0].activate_data = info;
+		menus = g_slist_prepend(menus, &efhd_menu_items[0]);
+		item = &efhd_menu_items[info->shown?2:1];
+		item->activate_data = info;
+		menus = g_slist_prepend(menus, item);
+	}
+
 	em_popup_add_items(emp, menus, (GDestroyNotify)g_slist_free);
 
 	menu = em_popup_create_menu_once(emp, target, target->mask, target->mask);
@@ -1212,8 +1247,10 @@ efhd_attachment_button(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObje
 
 	if (info->handle)
 		g_signal_connect(button, "clicked", G_CALLBACK(efhd_attachment_show), info);
-	else
+	else {
+		gtk_widget_set_sensitive(button, FALSE);
 		GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+	}
 
 	hbox = gtk_hbox_new(FALSE, 2);
 	info->forward = gtk_image_new_from_stock(GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_BUTTON);
@@ -1465,7 +1502,7 @@ efhd_format_attachment(EMFormat *emf, CamelStream *stream, CamelMimePart *part,
 	info->snoop_mime_type = emf->snoop_mime_type;
 
 	camel_stream_write_string(stream,
-				  "<table height=6><tr><td><a></a></td></tr></table>\n"
+				  EM_FORMAT_HTML_VPAD
 				  "<table cellspacing=0 cellpadding=0><tr><td>"
 				  "<table width=10 cellspacing=0 cellpadding=0>"
 				  "<tr><td></td></tr></table></td>");
@@ -1486,7 +1523,7 @@ efhd_format_attachment(EMFormat *emf, CamelStream *stream, CamelMimePart *part,
 
 	camel_stream_write_string(stream,
 				  "</font></td></tr><tr></table>\n"
-				  "<table height=6><tr><td><a></a></td></tr></table>\n");
+				  EM_FORMAT_HTML_VPAD);
 
 	if (handle) {
 		if (info->shown)
@@ -1497,7 +1534,7 @@ efhd_format_attachment(EMFormat *emf, CamelStream *stream, CamelMimePart *part,
 
 		classid = g_strdup_printf("bonobo-unknown:///em-format-html-display/%s", emf->part_id->str);
 		em_format_html_add_pobject((EMFormatHTML *)emf, sizeof(EMFormatHTMLPObject), classid, part, efhd_bonobo_object);
-		camel_stream_printf(stream, "<object classid=\"%s\" type=\"%s\">\n", classid, mime_type);
+		camel_stream_printf(stream, "<object classid=\"%s\" type=\"%s\"></object><br>>\n", classid, mime_type);
 	}
 
 	g_free(classid);
