@@ -26,15 +26,17 @@
 
 #include "e-shell-window-commands.h"
 
-#include "e-shell-about-box.h"
 #include "e-shell-importer.h"
 #include "e-shell-window.h"
 
 #include "evolution-shell-component-utils.h"
 
+#include "e-util/e-icon-factory.h"
 #include "e-util/e-dialog-utils.h"
 #include "e-util/e-passwords.h"
 
+#include <glib/gprintf.h>
+
 #include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-url.h>
 #include <libgnome/gnome-i18n.h>
@@ -44,6 +46,7 @@
 
 #include <bonobo/bonobo-ui-component.h>
 
+#include <string.h>
 
 /* Utility functions.  */
 
@@ -98,7 +101,6 @@ command_quit (BonoboUIComponent *uih,
 {
 	EShell *shell = e_shell_window_peek_shell (window);
 
-	e_shell_window_save_defaults (window);
 	e_shell_quit(shell);
 }
 
@@ -129,15 +131,203 @@ command_submit_bug (BonoboUIComponent *uih,
                 e_notice (NULL, GTK_MESSAGE_ERROR, _("Bug buddy could not be run."));
 }
 
-static int
-about_box_event_callback (GtkWidget *widget,
-			  GdkEvent *event,
-			  GtkWidget **widget_pointer)
-{
-	gtk_widget_destroy (GTK_WIDGET (*widget_pointer));
-	*widget_pointer = NULL;
+/* must be in utf8, the weird breaking of escaped strings
+   is so the hex escape strings dont swallow too many chars */
+static const char *authors[] = {
+	"Darin Adler",
+	"Arturo Espinosa Aldama",
+	"H\xC3\xA9" "ctor Garc\xC3\xAD" "a \xC3\x81" "lvarez",
+	"Jes\xC3\xBA" "s Bravo \xC3\x81" "lvarez",
+	"Seth Alves",
+	"Marius Andreiana",
+	"Sean Atkinson",
+	"Szabolcs BAN",
+	"Timur Bakeyev",
+	"Martin Baulig",
+	"Frank Belew",
+	"Dan Berger",
+	"Jacob Berkman",
+	"Matt Bissiri",
+	"Jonathan Blandford",
+	"Richard Boulton",
+	"Robert Brady",
+	"Kevin Breit",
+	"Martha Burke",
+	"Dave Camp",
+	"Ian Campbell",
+	"Anders Carlsson",
+	"Damon Chaplin",
+	"Abel Cheung",
+	"Zbigniew Chyla",
+	"Clifford R. Conover",
+	"Sam Creasey",
+	"Frederic Crozat",
+	"Wayne Davis",
+	"Rodney Dawes",
+	"Jos Dehaes",
+	"Fatih Demir",
+	"Arik Devens",
+	"Anna Marie Dirks",
+	"Bob Doan",
+	"Radek Doul\xC3\xADk",
+	"Edd Dumbill",
+	"Larry Ewing",
+	"Gilbert Fang",
+	"Francisco Javier F. Serrador",
+	"Nuno Ferreira",
+	"Valek Filippov",
+	"Nat Friedman",
+	"Sean Gao",
+	"Jeff Garzik",
+	"Nike Gerdts",
+	"Grzegorz Goawski",
+	"Jody Goldberg",
+	"Pablo Gonzalo del Campo",
+	"Mark Gordon",
+	"Kenny Graunke",
+	"Alex Graveley",
+	"Bertrand Guiheneuf",
+	"Jean-Noel Guiheneuf",
+	"Mikael Hallendal",
+	"Raja R Harinath",
+	"Heath Harrelson",
+	"Taylor Hayward",
+	"Jon K Hellan",
+	"Martin Hicks",
+	"Iain Holmes",
+	"Max Horn",
+	"Greg Hudson",
+	"Richard Hult",
+	"Andreas Hyden",
+	"Miguel de Icaza",
+	"Hans Petter Jansson",
+	"Jack Jia",
+	"Wang Jian",
+	"Sanshao Jiang",
+	"Benjamin Kahn",
+	"Yanko Kaneti",
+	"Lauris Kaplinski",
+	"Jeremy Katz",
+	"Mike Kestner",
+	"Christian Kreibich",
+	"Nicholas J Kreucher",
+	"Ronald Kuetemeier",
+	"Tuomas Kuosmanen",
+	"Mathieu Lacage",
+	"Christopher J. Lahey",
+	"Eneko Lacunza",
+	"Miles Lane",
+	"Jason Leach",
+	"Elliot Lee",
+	"Ji Lee",
+	"Timothy Lee",
+	"T\xC3\xB5" "ivo Leedj\xC3\xA4" "rv",
+	"Richard Li",
+	"Matthew Loper",
+	"Duarte Loreto",
+	"Harry Lu",
+	"Michael MacDonald",
+	"Duncan Mak",
+	"Kjartan Maraas",
+	"Garardo Marin",
+	"Matt Martin",
+	"Carlos Perell\xC3\xB3" " Mar\xC3\xAD" "n",
+	"Dietmar Maurer",
+	"William Jon McCann",
+	"Mike McEwan",
+	"Alastair McKinstry",
+	"Michael Meeks",
+	"Federico Mena",
+	"Christophe Merlet",
+	"Michael M. Morrison",
+	"Rodrigo Moya",
+	"Steve Murphy",
+	"Yukihiro Nakai",
+	"Martin Norb\xC3\xA4" "ck",
+	"Tomas Ogren",
+	"Ismael Olea",
+	"Eskil Heyn Olsen",
+	"Sergey Panov",
+	"Gediminas Paulauskas",
+	"Jesse Pavel",
+	"Havoc Pennington",
+	"Ettore Perazzoli",
+	"Petta Pietikainen",
+	"Herbert V. Riedel",
+	"Ariel Rios",
+	"JP Rosevear",
+	"Cody Russell",
+	"Martyn Russell",
+	"Changwoo Ryu",
+	"Pablo Saratxaga",
+	"Carsten Schaar",
+	"Joe Shaw",
+	"Timo Sirainen",
+	"Craig Small",
+	"Maciej Stachowiak",
+	"Jeffrey Stedfast",
+	"Jakub Steiner",
+	"Russell Steinthal",
+	"Vadim Strizhevsky",
+	"Yuri Syrota",
+	"Jason Tackaberry",
+	"Peter Teichman",
+	"Chris Toshok",
+	"Tom Tromey",
+	"Jon Trowbridge",
+	"Andrew T. Veliath",
+	"Gustavo Maciel Dias Vieira",
+	"Luis Villa",
+	"Stanislav Visnovsky",
+	"Aaron Weber",
+	"Dave West",
+	"Peter Williams",
+	"Matt Wilson",
+	"Matthew Wilson",
+	"Dan Winship",
+	"Jeremy Wise",
+	"Leon Zhang",
+	"Philip Zhao",
+	"Jukka Zitting",
+	"Michael Zucchi",
+	NULL
+};
+static const char *documentors[] = { 
+	"Aaron Weber",
+	"David Trowbridge",
+	NULL
+};
+static const char *translator_credits = "The GNOME Translation Project";
 
-	return TRUE;
+static GtkWidget *
+about_box_new (void)
+{
+	GtkWidget *about_box = NULL;
+	GdkPixbuf *pixbuf = NULL;
+	char copyright[1024];
+	char *filename = NULL;
+	
+	g_sprintf (copyright, "Copyright \xC2\xA9 1999 - 2004 Novell, Inc.");
+                                                                                
+	filename = g_build_filename (EVOLUTION_DATADIR, "pixmaps",
+				     "evolution-1.5.png", NULL);
+	if (filename != NULL) {
+		pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+		g_free (filename);
+	}
+                                                                                
+	about_box = gnome_about_new ("Evolution",
+				     VERSION,
+				     copyright,
+				     _("Groupware Suite"),
+				     authors, documentors,
+				     translator_credits,
+				     pixbuf);
+	
+        if (pixbuf != NULL)
+                g_object_unref (pixbuf);
+
+	return GTK_WIDGET (about_box);
 }
 
 static void
@@ -146,30 +336,19 @@ command_about_box (BonoboUIComponent *uih,
 		   const char *path)
 {
 	static GtkWidget *about_box_window = NULL;
-	GtkWidget *about_box;
 
 	if (about_box_window != NULL) {
 		gdk_window_raise (about_box_window->window);
 		return;
 	}
 
-	about_box = e_shell_about_box_new ();
-	gtk_widget_show (about_box);
-
-	about_box_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_type_hint (GTK_WINDOW (about_box_window), GDK_WINDOW_TYPE_HINT_DIALOG);
+	about_box_window = about_box_new ();
 	
-	gtk_window_set_resizable (GTK_WINDOW (about_box_window), FALSE);
-	g_signal_connect (about_box_window, "key_press_event",
-			  G_CALLBACK (about_box_event_callback), &about_box_window);
-	g_signal_connect (about_box_window, "button_press_event",
-			  G_CALLBACK (about_box_event_callback), &about_box_window);
-	g_signal_connect (about_box_window, "delete_event",
-			  G_CALLBACK (about_box_event_callback), &about_box_window);
+	g_signal_connect (G_OBJECT (about_box_window), "destroy",
+			  G_CALLBACK (gtk_widget_destroyed), &about_box_window);
 
 	gtk_window_set_transient_for (GTK_WINDOW (about_box_window), GTK_WINDOW (window));
-	gtk_window_set_title (GTK_WINDOW (about_box_window), _("About Ximian Evolution"));
-	gtk_container_add (GTK_CONTAINER (about_box_window), about_box);
+
 	gtk_widget_show (about_box_window);
 }
 
@@ -178,7 +357,9 @@ command_help_faq (BonoboUIComponent *uih,
 		  EShellWindow *window,
 		  const char *path)
 {
-	gnome_url_show ("http://www.ximian.com/apps/evolution-faq.html", NULL);	/* FIXME use the error */
+	/* FIXME Show when we have a faq
+	/* FIXME use the error */
+	gnome_url_show ("http://gnome.org/projects/evolution/faq.shtml", NULL);	
 }
 
 static void
@@ -188,7 +369,6 @@ command_quick_reference (BonoboUIComponent *uih,
 {
 	char *quickref;
 	char *uri;
-	char *mimetype;
 	char *command;
 	GString *str;
 	GnomeVFSMimeApplication *app;
@@ -205,35 +385,37 @@ command_quick_reference (BonoboUIComponent *uih,
 
 		quickref = g_build_filename (EVOLUTION_HELPDIR, "quickref", lang, "quickref.pdf", NULL);
 		if (g_file_test (quickref, G_FILE_TEST_EXISTS)) {
-			mimetype = gnome_vfs_get_mime_type (quickref);
-			app = gnome_vfs_mime_get_default_application (mimetype);
-			str = g_string_new ("");
-			str = g_string_append (str, app->command);
+			app = gnome_vfs_mime_get_default_application ("application/pdf");
+			if (app) {
+				str = g_string_new ("");
+				str = g_string_append (str, app->command);
 
-			switch (app->expects_uris) {
-			case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS:
-				uri = g_strconcat ("file://", quickref, NULL);
-				g_string_append_printf (str, " %s", uri);
-				g_free (uri);
-				break;
-			case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS:
-			case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS_FOR_NON_FILES:
-				g_string_append_printf (str, " %s", quickref);
-				break;
+				switch (app->expects_uris) {
+				case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS:
+					uri = g_strconcat ("file://", quickref, NULL);
+					g_string_append_printf (str, " %s", uri);
+					g_free (uri);
+					break;
+				case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS:
+				case GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS_FOR_NON_FILES:
+					g_string_append_printf (str, " %s", quickref);
+					break;
+				}
+
+				command = g_string_free (str, FALSE);
+				if (command != NULL &&
+				!g_spawn_command_line_async (command, NULL)) {
+					g_warning ("Could not launch %s", command);
+				}
+
+				g_free (command);
+				gnome_vfs_mime_application_free (app);
 			}
 
-			command = g_string_free (str, FALSE);
-			if (command != NULL &&
-			    !g_spawn_command_line_async (command, NULL)) {
-				g_warning ("Could not launch %s", command);
-			}
-			g_free (command);
-
-			gnome_vfs_mime_application_free (app);
-			g_free (mimetype);
 			g_free (quickref);
 			return;
 		}
+
 		g_free (quickref);
 	}
 }
@@ -335,7 +517,6 @@ static BonoboUIVerb tools_verbs[] = {
 };
 
 static BonoboUIVerb help_verbs [] = {
-	BONOBO_UI_VERB ("HelpFAQ", (BonoboUIVerbFn) command_help_faq),
 	BONOBO_UI_VERB ("QuickReference", (BonoboUIVerbFn) command_quick_reference),
 	BONOBO_UI_VERB ("HelpSubmitBug", (BonoboUIVerbFn) command_submit_bug),
 	BONOBO_UI_VERB ("HelpAbout", (BonoboUIVerbFn) command_about_box),
@@ -344,22 +525,22 @@ static BonoboUIVerb help_verbs [] = {
 };
 
 static EPixmap pixmaps [] = {
-	E_PIXMAP ("/commands/SendReceive", "send-receive.xpm"),
-	E_PIXMAP ("/Toolbar/SendReceive", "buttons/send-24-receive.png"),
-	E_PIXMAP ("/menu/File/FileImporter", "import.xpm"),
-	E_PIXMAP ("/menu/File/ToggleOffline", "work_offline.xpm"),
-	E_PIXMAP ("/menu/Tools/Settings", "settings-16.png"),
-
+	E_PIXMAP ("/commands/SendReceive", "stock_mail-send-receive", E_ICON_SIZE_MENU),
+	E_PIXMAP ("/Toolbar/SendReceive", "stock_mail-send-receive", E_ICON_SIZE_LARGE_TOOLBAR),
+	E_PIXMAP ("/menu/File/FileImporter", "stock_mail-import", E_ICON_SIZE_MENU),
+	E_PIXMAP ("/menu/File/ToggleOffline", "stock_disconnect", E_ICON_SIZE_MENU),
+	E_PIXMAP ("/menu/Tools/Settings", "gnome-settings", E_ICON_SIZE_MENU),
+	
 	E_PIXMAP_END
 };
 
 static EPixmap offline_pixmaps [] = {
-	E_PIXMAP ("/menu/File/ToggleOffline", "work_offline.xpm"),
+	E_PIXMAP ("/menu/File/ToggleOffline", "stock_disconnect", E_ICON_SIZE_MENU),
 	E_PIXMAP_END
 };
 
 static EPixmap online_pixmaps [] = {
-	E_PIXMAP ("/menu/File/ToggleOffline", "work_online-16.png"),
+	E_PIXMAP ("/menu/File/ToggleOffline", "stock_connect", E_ICON_SIZE_MENU),
 	E_PIXMAP_END
 };
 
@@ -427,6 +608,21 @@ shell_line_status_changed_cb (EShell *shell,
 	update_offline_menu_item (shell_window, new_status);
 }
 
+static void
+view_toolbar_item_toggled_handler (BonoboUIComponent           *ui_component,
+				   const char                  *path,
+				   Bonobo_UIComponent_EventType type,
+				   const char                  *state,
+				   EShellWindow                *shell_window)
+{
+	gboolean is_visible;
+
+	is_visible = state[0] == '1';
+
+	bonobo_ui_component_set_prop (ui_component, "/Toolbar",
+				      "hidden", is_visible ? "0" : "1", NULL);
+}
+
 
 /* Public API.  */
 
@@ -447,6 +643,9 @@ e_shell_window_commands_setup (EShellWindow *shell_window)
 	bonobo_ui_component_add_verb_list_with_data (uic, actions_verbs, shell_window);
 	bonobo_ui_component_add_verb_list_with_data (uic, tools_verbs, shell_window);
 	bonobo_ui_component_add_verb_list_with_data (uic, help_verbs, shell_window);
+	bonobo_ui_component_add_listener (uic, "ViewToolbar",
+					  (BonoboUIListenerFn)view_toolbar_item_toggled_handler,
+					  (gpointer)shell_window);
 
 	e_pixmaps_update (uic, pixmaps);
 
