@@ -128,6 +128,7 @@ e_cert_db_dispose (GObject *object)
 		G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+#if notyet
 PRBool
 ucs2_ascii_conversion_fn (PRBool toUnicode,
 			  unsigned char *inBuf,
@@ -139,6 +140,7 @@ ucs2_ascii_conversion_fn (PRBool toUnicode,
 {
 	printf ("in ucs2_ascii_conversion_fn\n");
 }
+#endif
 
 static char* PR_CALLBACK
 pk11_password (PK11SlotInfo* slot, PRBool retry, void* arg)
@@ -155,6 +157,9 @@ pk11_password (PK11SlotInfo* slot, PRBool retry, void* arg)
 		       &pwd,
 		       &rv);
 
+	if (pwd == NULL)
+		return NULL;
+
 	nsspwd = PORT_Strdup (pwd);
 	memset (pwd, 0, strlen (pwd));
 	g_free (pwd);
@@ -200,7 +205,9 @@ initialize_nss (void)
 	SEC_PKCS12EnableCipher(PKCS12_DES_56, 1);
 	SEC_PKCS12EnableCipher(PKCS12_DES_EDE3_168, 1);
 	SEC_PKCS12SetPreferredCipher(PKCS12_DES_EDE3_168, 1);
+#if notyet
 	PORT_SetUCS2_ASCIIConversionFunction(ucs2_ascii_conversion_fn);
+#endif
 }
 
 static void
@@ -374,12 +381,12 @@ e_cert_db_find_cert_by_nickname (ECertDB *certdb,
 	}
 }
 
+#if notyet
 ECert*
 e_cert_db_find_cert_by_key (ECertDB *certdb,
 			    const char *db_key,
 			    GError **error)
 {
-#if 0
 	/*  nsNSSShutDownPreventionLock locker;*/
 	SECItem keyItem = {siBuffer, NULL, 0};
 	SECItem *dummy;
@@ -415,7 +422,6 @@ e_cert_db_find_cert_by_key (ECertDB *certdb,
 
 	/* XXX gerror */
 	return NULL;
-#endif
 }
 
 GList*
@@ -438,6 +444,7 @@ e_cert_db_find_email_signing_cert (ECertDB *certdb,
 				   GError **error)
 {
 }
+#endif
 
 ECert*
 e_cert_db_find_cert_by_email_address (ECertDB *certdb,
@@ -1112,6 +1119,7 @@ e_cert_db_import_pkcs12_file (ECertDB *cert_db,
 			      const char *file_path,
 			      GError **error)
 {
+#if notyet
 	EPKCS12 *pkcs12 = e_pkcs12_new ();
 	GError *e = NULL;
 
@@ -1121,6 +1129,10 @@ e_cert_db_import_pkcs12_file (ECertDB *cert_db,
 	}
 
 	return TRUE;
+#else
+#warning "e_cert_db_import_pkcs12_file not implemented yet"
+	return FALSE;
+#endif
 }
 
 gboolean
@@ -1157,6 +1169,7 @@ e_cert_db_login_to_slot (ECertDB *cert_db,
 			PK11_InitPin (slot, "", pwd);
 		}
 
+		PK11_SetPasswordFunc(pk11_password);
 		if (PK11_Authenticate (slot, PR_TRUE, NULL) != SECSuccess) {
 			printf ("PK11_Authenticate failed (err = %d/%d)\n", PORT_GetError(), PORT_GetError() + 0x2000);
 			return FALSE;
